SystemOrganization addCategory: #'Math-DHB-Numerical'!
SystemOrganization addCategory: 'Math-DHB-Numerical-Math-Eigenvalues'!
SystemOrganization addCategory: 'Math-DHB-Numerical-Math-Matrix'!

!Number class methodsFor: '*Math-DHB-Numerical' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
random
		"Answers a random number between 0 and the receiver
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/2/99 "
	^ DhbMitchellMooreGenerator new floatValue! !

!Number methodsFor: '*Math-DHB-Numerical' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
addPolynomial: aPolynomial
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 19/4/99 "
	^aPolynomial addNumber: self! !

!Number methodsFor: '*Math-DHB-Numerical' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
beta: aNumber
		"Computes the beta function of the receiver and aNumber
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 1/3/99 "
	^( self logBeta: aNumber) exp! !

!Number methodsFor: '*Math-DHB-Numerical' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
dividingPolynomial: aPolynomial
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/4/99 "
	^aPolynomial timesNumber: (1 / self)! !

!Number methodsFor: '*Math-DHB-Numerical' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
equalsTo: aNumber
		"compare to Float>>closeTo:
		
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 21/4/99 "
	^self relativelyEqualsTo: aNumber upTo: DhbFloatingPointMachine new defaultNumericalPrecision! !

!Number methodsFor: '*Math-DHB-Numerical' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
errorFunction
		"Answer the error function for the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^ DhbErfApproximation new value: self! !

!Number methodsFor: '*Math-DHB-Numerical' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
gamma
		"Compute the Gamma function for the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^ self > 1
		ifTrue: [ ^ DhbLanczosFormula new gamma: self]
		ifFalse:[ self < 0
						ifTrue: [ Float pi / ( ( Float pi * self) sin * ( 1 - self) gamma)]
						ifFalse:[ ( DhbLanczosFormula new gamma: ( self + 1)) / self]
					]! !

!Number methodsFor: '*Math-DHB-Numerical' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
logBeta: aNumber
		"Computes the logarithm of the beta function of the receiver and aNumber
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 1/3/99 "
	^self logGamma + aNumber logGamma - ( self + aNumber) logGamma! !

!Number methodsFor: '*Math-DHB-Numerical' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
logGamma
		"Computes the log of the Gamma function (for positive numbers only)
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 1/3/99 "
	^self > 1
		ifTrue: [ DhbLanczosFormula new logGamma: self]
		ifFalse:[ self > 0
						ifTrue: [ ( DhbLanczosFormula new logGamma: ( self + 1) ) - self ln ]
						ifFalse: [ ^self error: 'Argument for the log gamma function must be positive']
					]! !

!Number methodsFor: '*Math-DHB-Numerical' stamp: 'WernerKassens 5/2/2014 09:10'!
productWithMatrix: aMatrix
^aMatrix class rows: (aMatrix rowsCollect: [:r| self productWithVector: r])! !

!Number methodsFor: '*Math-DHB-Numerical' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
productWithVector: aVector
		"Answers a new vector product of the receiver with aVector.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^aVector collect: [ :each | each * self]! !

!Number methodsFor: '*Math-DHB-Numerical' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
random
		"Answers a random number distributed between 0 and the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/2/99 "
	^self class random * self! !

!Number methodsFor: '*Math-DHB-Numerical' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
relativelyEqualsTo: aNumber upTo: aSmallNumber
		"compare to Float>>closeTo:
		generally called from Number>>equalsTo:
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 21/4/99 "
	| norm |
	norm := self abs max: aNumber abs.
	^norm <= DhbFloatingPointMachine new defaultNumericalPrecision
		or: [ (self - aNumber) abs < ( aSmallNumber * norm)]! !

!Number methodsFor: '*Math-DHB-Numerical' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
subtractToPolynomial: aPolynomial
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 19/4/99 "
	^aPolynomial addNumber: self negated! !

!Number methodsFor: '*Math-DHB-Numerical' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
timesPolynomial: aPolynomial
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/4/99 "
	^aPolynomial timesNumber: self! !

DhbProbabilityDensity subclass: #DhbCauchyDistribution
	instanceVariableNames: 'mu beta'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbCauchyDistribution class methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
distributionName
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^'Cauchy distribution'! !

!DhbCauchyDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
fromHistogram: aHistogram 
	"Create an instance of the receiver with parameters estimated from the
		  given histogram using best guesses. This method can be used to
		  find the initial values for a fit.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 3/3/99 "

	^self shape: aHistogram average
		scale: 4 * aHistogram variance 
				/ (Float pi * (aHistogram maximum squared + aHistogram minimum squared)) 
						sqrt! !

!DhbCauchyDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
new
	"Create an instance of the receiver with center 0 and scale 1.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "

	^self shape: 0 scale: 1! !

!DhbCauchyDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
shape: aNumber1 scale: aNumber2 
	"Create an instance of the receiver with given center and scale parameters.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "

	^super new initialize: aNumber1 scale: aNumber2! !

!DhbCauchyDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
average
		"Answer the average of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^mu! !

!DhbCauchyDistribution methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
changeParametersBy: aVector
		"Modify the parameters of the receiver by aVector.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/3/99 "
	mu := mu + ( aVector at: 1).
	beta := beta + ( aVector at: 2).! !

!DhbCauchyDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
distributionValue: aNumber
		"Answers the probability of observing a random variable distributed according to
		 the receiver with a value lower than or equal to aNumber.
		 Assumes that the value of the receiver is 0 for x < 0.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^(( aNumber - mu) / beta) arcTan / Float pi + (1 / 2)! !

!DhbCauchyDistribution methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialize: aNumber1 scale: aNumber2
		"Private - Initialize the parameters of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	mu := aNumber1.
	beta := aNumber2.
	^self! !

!DhbCauchyDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
parameters
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^Array with: mu with: beta! !

!DhbCauchyDistribution methodsFor: 'information' stamp: 'WernerKassens 9/16/2015 16:44'!
privateInverseDistributionValue: aNumber
		"Private - Answer the number whose acceptance is aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^aNumber =0 
		ifTrue:[Float infinity negated] 
		ifFalse: [aNumber =1 
			ifTrue:[Float infinity ] 
			ifFalse:[( ( aNumber - (1 / 2)) * Float pi) tan * beta + mu]].! !

!DhbCauchyDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
standardDeviation
		"The standard deviation of the receiver is not defined.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^nil! !

!DhbCauchyDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
value: aNumber
		"Answers the probability that a random variable distributed according to the receiver
		 gives a value between aNumber and aNumber + espilon (infinitesimal interval).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^beta / ( Float pi * ( beta squared + ( aNumber - mu) squared))! !

!DhbCauchyDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
valueAndGradient: aNumber
		"Answers an Array containing the value of the receiver at aNumber
		 and the gradient of the receiver's respective to the receiver's
		 parameters evaluated at aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/3/99 "
	| dp denominator |
	dp := self value: aNumber.
	denominator := 1 / ( ( aNumber - mu) squared + beta squared).
	^Array with: dp
		   with: ( DhbVector with: 2 * dp * ( aNumber - mu) * denominator
							 with: dp * ( 1 / beta - ( 2 * beta * denominator)))! !

!DhbCauchyDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
variance
		"The variance of the receiver is not defined.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^nil! !

DhbProbabilityDensity subclass: #DhbExponentialDistribution
	instanceVariableNames: 'beta'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbExponentialDistribution class methodsFor: 'information' stamp: 'SergeStinckwich 3/8/2015 21:33'!
distributionName
	^ 'Exponential distribution'! !

!DhbExponentialDistribution class methodsFor: 'creation' stamp: 'SergeStinckwich 3/8/2015 21:33'!
fromHistogram: aHistogram
		"Create an instance of the receiver with parameters estimated from the
		  given histogram using best guesses. This method can be used to
		  find the initial values for a fit."
	| mu |
	aHistogram minimum < 0
		ifTrue: [ ^nil].
	mu := aHistogram average.
	^mu > 0 ifTrue: [ self scale: aHistogram average]
			ifFalse:[ nil]! !

!DhbExponentialDistribution class methodsFor: 'creation' stamp: 'SergeStinckwich 3/8/2015 21:33'!
new
		"Create a new instance of the receiver with scale parameter 1."
	^super new initialize: 1! !

!DhbExponentialDistribution class methodsFor: 'creation' stamp: 'SergeStinckwich 3/8/2015 21:33'!
scale: aNumber
		"Create a new instance of the receiver with given scale parameter."
	^super new initialize: aNumber! !

!DhbExponentialDistribution methodsFor: 'information' stamp: 'SergeStinckwich 3/8/2015 21:31'!
average
		"Answer the average of the receiver."
	^beta! !

!DhbExponentialDistribution methodsFor: 'transformation' stamp: 'SergeStinckwich 3/8/2015 21:31'!
changeParametersBy: aVector
		"Modify the parameters of the receiver by aVector."
	beta := beta + ( aVector at: 1).! !

!DhbExponentialDistribution methodsFor: 'information' stamp: 'SergeStinckwich 3/8/2015 21:31'!
distributionValue: aNumber
		"Answers the probability of observing a random variable distributed according to
		 the receiver with a value lower than or equal to aNumber."

	^[1 - ( ( aNumber / beta negated) exp)]
			on: Error do: [ :signal | signal return: 0]! !

!DhbExponentialDistribution methodsFor: 'initialization' stamp: 'SergeStinckwich 3/8/2015 21:31'!
initialize: aNumber
		"Private - Set the scale parameter of the receiver to aNumber."
	aNumber > 0
		ifFalse: [ self error: 'Illegal distribution parameters'].
	beta := aNumber.
	^self! !

!DhbExponentialDistribution methodsFor: 'information' stamp: 'SergeStinckwich 3/8/2015 21:31'!
kurtosis
		"Answer the kurtosis of the receiver."
	^6! !

!DhbExponentialDistribution methodsFor: 'information' stamp: 'SergeStinckwich 3/8/2015 21:32'!
parameters
	^ Array with: beta! !

!DhbExponentialDistribution methodsFor: 'information' stamp: 'SergeStinckwich 3/8/2015 21:32'!
privateInverseDistributionValue: aNumber
		"Private - Answer the number whose acceptance is aNumber."
	^(1 - aNumber) ln negated * beta! !

!DhbExponentialDistribution methodsFor: 'information' stamp: 'SergeStinckwich 3/8/2015 21:32'!
random
		"Answer a random number distributed according to the receiver."
	^DhbMitchellMooreGenerator new floatValue ln * beta negated! !

!DhbExponentialDistribution methodsFor: 'information' stamp: 'SergeStinckwich 3/8/2015 21:32'!
skewness
		"Answer the skewness of the receiver."
	^2! !

!DhbExponentialDistribution methodsFor: 'information' stamp: 'SergeStinckwich 3/8/2015 21:32'!
standardDeviation
		"Answer the standard deviation of the receiver."
	^beta! !

!DhbExponentialDistribution methodsFor: 'information' stamp: 'SergeStinckwich 3/8/2015 21:32'!
value: aNumber
		"Answers the probability that a random variable distributed according to the receiver
		 gives a value between aNumber and aNumber + espilon (infinitesimal interval)."

	^[ ( aNumber / beta) negated exp / beta]
			on: Error do: [ :signal | signal return: 0]! !

!DhbExponentialDistribution methodsFor: 'information' stamp: 'SergeStinckwich 3/8/2015 21:32'!
valueAndGradient: aNumber
		"Answers an Array containing the value of the receiver at aNumber
		 and the gradient of the receiver's respective to the receiver's
		 parameters evaluated at aNumber."
	| dp |
	dp := self value: aNumber.
	^Array with: dp
		   with: ( DhbVector with: ( aNumber / beta - 1) * dp / beta)! !

DhbProbabilityDensity subclass: #DhbFisherTippettDistribution
	instanceVariableNames: 'alpha beta'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbFisherTippettDistribution class methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
distributionName
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^'Fisher-Tippett distribution'! !

!DhbFisherTippettDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
fromHistogram: aHistogram 
	"Create an instance of the receiver with parameters estimated from the
		  given histogram using best guesses. This method can be used to
		  find the initial values for a fit.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 3/3/99 "

	| beta |
	beta := aHistogram standardDeviation.
	beta = 0 ifTrue: [^nil].
	beta := beta * (6 sqrt / Float pi).
	^self shape: aHistogram average - (0.5772156649 * beta) scale: beta! !

!DhbFisherTippettDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
new
	"Create a standard version of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "

	^self shape: 0 scale: 1! !

!DhbFisherTippettDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
shape: aNumber1 scale: aNumber2 
	"Create an instance of the receiver with given shape and scale parameters.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "

	^super new initialize: aNumber1 scale: aNumber2! !

!DhbFisherTippettDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
average
		"Answer the average of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^0.577256649 * beta + alpha! !

!DhbFisherTippettDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
changeParametersBy: aVector
		"Modify the parameters of the receiver by aVector.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/3/99 "
	alpha := alpha + ( aVector at: 1).
	beta := beta + ( aVector at: 2).! !

!DhbFisherTippettDistribution methodsFor: 'information' stamp: 'WernerKassens 9/10/2015 09:23'!
distributionValue: aNumber
		"Answers the probability of observing a random variable distributed according to
		 the receiver with a value lower than or equal to aNumber."
	| arg |
	arg := ( aNumber - alpha) / beta.
	arg := arg < DhbFloatingPointMachine new largestExponentArgument negated
					ifTrue: [ ^0]
					ifFalse:[arg negated exp].
	^arg negated exp! !

!DhbFisherTippettDistribution methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialize: aNumber1 scale: aNumber2
		"Private - Initialize the parameters of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	aNumber2 > 0
		ifFalse: [ self error: 'Illegal distribution parameters'].
	alpha := aNumber1.
	beta := aNumber2.
	^self! !

!DhbFisherTippettDistribution methodsFor: 'information' stamp: 'wernerkassens 10/30/2014 11:01'!
integralFrom: aNumber1 to: aNumber2
		"Private - Compute the integral of the receiver from aNumber1 to aNumber2.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 14/4/99 "
	^( DhbRombergIntegrator function: [:x|self value:x] from: aNumber1 to: aNumber2) evaluate! !

!DhbFisherTippettDistribution methodsFor: 'information' stamp: 'wernerkassens 10/30/2014 10:56'!
integralUpTo: aNumber
		"Private - Compute the integral of the receiver from -infinity to aNumber.
		 aNumber must be below 0 (no checking!!).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 14/4/99 "
	^( DhbRombergIntegrator function:
			[ :x | x = 0 ifTrue: [ 0] ifFalse: [ ( self value: 1 / x) / x squared] ]
			from: 1 / aNumber to: 0) evaluate! !

!DhbFisherTippettDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
kurtosis
		"Answer the kurtosis of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^2.4! !

!DhbFisherTippettDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
parameters
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^Array with: alpha with: beta! !

!DhbFisherTippettDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
random
		"Answer a random number distributed according to the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/4/99 "
	| t |
	[ t := DhbMitchellMooreGenerator new floatValue ln negated.
	  t > 0] whileFalse: [].
	^t ln negated * beta + alpha! !

!DhbFisherTippettDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
skewness
		"Answer the skewness of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^1.3! !

!DhbFisherTippettDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
standardDeviation
		"Answer the standard deviation of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^Float pi * beta / ( 6 sqrt)! !

!DhbFisherTippettDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
value: aNumber
		"Answers the probability that a random variable distributed according to the receiver
		 gives a value between aNumber and aNumber + espilon (infinitesimal interval).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	| arg |
	arg := ( aNumber - alpha) / beta.
	arg := arg > DhbFloatingPointMachine new largestExponentArgument ifTrue: [ ^0]
						ifFalse:[arg negated exp + arg].
	^arg > DhbFloatingPointMachine new largestExponentArgument ifTrue: [ 0]
						ifFalse:[ arg negated exp / beta]! !

!DhbFisherTippettDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
valueAndGradient: aNumber
		"Answers an Array containing the value of the receiver at aNumber
		 and the gradient of the receiver's respective to the receiver's
		 parameters evaluated at aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/3/99 "
	| dp dy y|
	dp := self value: aNumber.
	y := ( aNumber - alpha) / beta.
	dy := ( y negated exp - 1).
	^Array with: dp
		   with: ( DhbVector with: dy * dp / beta negated
							 with: dp * ( y * dy + 1) / beta negated)! !

DhbProbabilityDensity subclass: #DhbHistogrammedDistribution
	instanceVariableNames: 'histogram'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbHistogrammedDistribution class methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
distributionName
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^'Experimental distribution'! !

!DhbHistogrammedDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
histogram: aHistogram
		"Create a new instance of the receiver corresponding to a histogram.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/2/99 "
	^super new initialize: aHistogram! !

!DhbHistogrammedDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
new
		"Prevent using this message to create instances
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^self error: 'Illegal creation message for this class'! !

!DhbHistogrammedDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
acceptanceBetween: aNumber1 and: aNumber2
		"Answers the probability of observing a random variable distributed according to
		 the receiver with a value larger than aNumber 1 and lower than or equal to aNumber2.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^( histogram countsBetween: ( aNumber1 max: histogram minimum)
						and: ( aNumber2 min: histogram maximum) ) / histogram totalCount! !

!DhbHistogrammedDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
average
		"Answer the average of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^histogram average! !

!DhbHistogrammedDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
distributionValue: aNumber
		"Answers the probability of observing a random variable distributed according to
		 the receiver with a value lower than or equal to aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^aNumber < histogram minimum
		ifTrue: [ 0]
		ifFalse:[ aNumber < histogram maximum
							ifTrue: [ ( histogram countsUpTo: aNumber) / histogram totalCount]
							ifFalse:[ 1]
					]! !

!DhbHistogrammedDistribution methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialize: aHistogram
		"Private - Defines the histogram of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/2/99 "
	aHistogram count = 0
		ifTrue: [ self error: 'Cannot define probability density on an empty histogram'].
	histogram := aHistogram.
	^self! !

!DhbHistogrammedDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
kurtosis
		"Answer the kurtosis of the receiver.
		 Undefined. Must be implemented by subclass.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^histogram kurtosis! !

!DhbHistogrammedDistribution methodsFor: 'information' stamp: 'WernerKassens 4/9/2014 14:24'!
parameters 
^nil! !

!DhbHistogrammedDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
privateInverseDistributionValue: aNumber
		"Private - Answer the number whose distribution is aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^histogram inverseDistributionValue: aNumber! !

!DhbHistogrammedDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
skewness
		"Answer the skewness of the receiver.
		 Undefined. Must be implemented by subclass.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^histogram skewness! !

!DhbHistogrammedDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
standardDeviation
		"Answer the standard deviation of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^histogram standardDeviation! !

!DhbHistogrammedDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
value: aNumber
		"Answers the probability that a random variable distributed according to the receiver
		 gives a value between aNumber and aNumber + espilon (infinitesimal interval).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 17/2/99 "
	^( aNumber >= histogram minimum and: [ aNumber < histogram maximum])
		ifTrue: [ ( histogram countAt: aNumber) / ( histogram totalCount * histogram binWidth)]
		ifFalse:[ 0]! !

!DhbHistogrammedDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
variance
		"Answer the variance of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^histogram variance! !

DhbProbabilityDensity subclass: #DhbLaplaceDistribution
	instanceVariableNames: 'mu beta'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbLaplaceDistribution class methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
distributionName
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^'Laplace distribution'! !

!DhbLaplaceDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
fromHistogram: aHistogram 
	"Create an instance of the receiver with parameters estimated from the
		  given histogram using best guesses. This method can be used to
		  find the initial values for a fit.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 3/3/99 "

	^self shape: aHistogram average scale: (aHistogram variance / 2) sqrt! !

!DhbLaplaceDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
new
	"
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "

	^self shape: 0 scale: 1! !

!DhbLaplaceDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
shape: aNumber1 scale: aNumber2 
	"
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "

	^super new initialize: aNumber1 scale: aNumber2! !

!DhbLaplaceDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
average
		"Answer the average of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^mu! !

!DhbLaplaceDistribution methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
changeParametersBy: aVector
		"Modify the parameters of the receiver by aVector.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/3/99 "
	mu := mu + ( aVector at: 1).
	beta := beta + ( aVector at: 2).! !

!DhbLaplaceDistribution methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
distributionValue: aNumber
		"Answers the probability of observing a random variable distributed according to
		 the receiver with a value lower than or equal to aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^aNumber > mu
		ifTrue: [ 1 - ( ( ( aNumber - mu) / beta) negated exp / 2)]
		ifFalse:[ ( ( ( aNumber - mu) / beta) exp / 2)]! !

!DhbLaplaceDistribution methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialize: aNumber1 scale: aNumber2
		"Private - Initialize the parameters of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	mu := aNumber1.
	beta := aNumber2.
	^self! !

!DhbLaplaceDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
kurtosis
		"Answer the kurtosis of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^3! !

!DhbLaplaceDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
parameters
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^Array with: mu with: beta! !

!DhbLaplaceDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
random
		"Answer a random number distributed accroding to the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	| r |
	r := DhbMitchellMooreGenerator new floatValue ln * beta negated.
	^DhbMitchellMooreGenerator new floatValue > 0.5
		ifTrue: [ mu + r]
		ifFalse:[ mu - r]! !

!DhbLaplaceDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
skewness
		"Answer the skewness of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^0! !

!DhbLaplaceDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
standardDeviation
		"Answer the standard deviation of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^beta * ( 2 sqrt)! !

!DhbLaplaceDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
value: aNumber
		"Answers the probability that a random variable distributed according to the receiver
		 gives a value between aNumber and aNumber + espilon (infinitesimal interval).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^( ( aNumber - mu) / beta) abs negated exp / ( 2 * beta)! !

!DhbLaplaceDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
valueAndGradient: aNumber
		"Answers an Array containing the value of the receiver at aNumber
		 and the gradient of the receiver's respective to the receiver's
		 parameters evaluated at aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/3/99 "
	| dp |
	dp := self value: aNumber.
	^Array	with: dp
			with: ( DhbVector with: ( aNumber - mu) sign * dp / beta
							  with: ( ( ( aNumber - mu) abs / beta - 1) * dp / beta))! !

DhbProbabilityDensity subclass: #DhbProbabilityDensityWithUnknownDistribution
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

DhbProbabilityDensityWithUnknownDistribution subclass: #DhbLogNormalDistribution
	instanceVariableNames: 'normalDistribution'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbLogNormalDistribution class methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
distributionName
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^'Log normal distribution'! !

!DhbLogNormalDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
fromHistogram: aHistogram
		"Create an instance of the receiver with parameters estimated from the
		  given histogram using best guesses. This method can be used to
		  find the initial values for a fit.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 3/3/99 "
	| average variance sigma2 |
	aHistogram minimum < 0
		ifTrue: [ ^nil].
	average := aHistogram average.
	average > 0
		ifFalse: [ ^nil].
	variance := aHistogram variance.
	sigma2 := ( variance / average squared + 1) ln.
	sigma2 > 0
		ifFalse: [ ^nil].
	^self new: ( average ln - (sigma2 * 0.5)) sigma: sigma2 sqrt! !

!DhbLogNormalDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
new
		"Create a new instance of the receiver with mu=0 and sigma=1.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^self new: 0 sigma: 1! !

!DhbLogNormalDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
new: aNumber1 sigma: aNumber2
		"Create a new instance of the receiver with given mu and sigma.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^super new initialize: aNumber1 sigma: aNumber2! !

!DhbLogNormalDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
average
		"Answer the average of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^( normalDistribution variance*0.5 + normalDistribution average) exp! !

!DhbLogNormalDistribution methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
changeParametersBy: aVector
		"Modify the parameters of the receiver by aVector.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/3/99 "
	normalDistribution changeParametersBy: aVector.! !

!DhbLogNormalDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
fourthCentralMoment
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 24/6/99 "
	| y x |
	y := normalDistribution average exp.
	x := normalDistribution variance exp.
	^( y squared squared) * ( x squared)
		* ( ( ( x squared * x - 4) * ( x squared) + 6) * x - 3)! !

!DhbLogNormalDistribution methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialize: aNumber1 sigma: aNumber2
		"Private - Defines the parameters of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	normalDistribution := DhbNormalDistribution new: aNumber1 sigma: aNumber2.
	^self! !

!DhbLogNormalDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
kurtosis
		"Answer the variance of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	| x |
	x := normalDistribution variance exp.
	^( ( x + 2) * x + 3) * ( x squared) - 6! !

!DhbLogNormalDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
parameters
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^normalDistribution parameters! !

!DhbLogNormalDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
random
		"Answer a random number distributed accroding to the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^normalDistribution random exp! !

!DhbLogNormalDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
skewness
		"Answer the variance of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	| x |
	x := normalDistribution variance exp.
	^(x - 1) sqrt * (x + 2)! !

!DhbLogNormalDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
thirdCentralMoment
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 24/6/99 "
	| y x |
	y := normalDistribution average exp.
	x := normalDistribution variance exp.
	^( y squared * y) * ( x raisedTo: (3/2))
		* ( ( x squared negated + 3) * x - 2)! !

!DhbLogNormalDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
value: aNumber
		"Answers the probability that a random variable distributed according to the receiver
		 gives a value between aNumber and aNumber + espilon (infinitesimal interval).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^aNumber > 0
		ifTrue: [ ( normalDistribution value: aNumber ln) / aNumber]
		ifFalse:[ 0]! !

!DhbLogNormalDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
variance
		"Answer the variance of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^( normalDistribution average * 2 + normalDistribution variance) exp * ( normalDistribution variance exp - 1)! !

!DhbProbabilityDensityWithUnknownDistribution methodsFor: 'information' stamp: 'wernerkassens 10/30/2014 11:39'!
acceptanceBetween: aNumber1 and: aNumber2
		"Answers the probability of observing a random variable distributed according to
		 the receiver with a value larger than aNumber 1 and lower than or equal to aNumber2.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^( DhbRombergIntegrator function: [:x| self value:x] from: aNumber1 to: aNumber2) evaluate! !

!DhbProbabilityDensityWithUnknownDistribution methodsFor: 'information' stamp: 'wernerkassens 10/30/2014 11:38'!
distributionValue: aNumber
		"Answers the probability of observing a random variable distributed according to
		 the receiver with a value lower than or equal to aNumber.
		 This general purpose routine uses numerical integration.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^( DhbRombergIntegrator function: [:x| self value:x] from: self lowestValue to: aNumber) evaluate! !

!DhbProbabilityDensityWithUnknownDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
lowestValue
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 1/18/00 "
	^0! !

DhbProbabilityDensity subclass: #DhbTriangularDistribution
	instanceVariableNames: 'lowLimit highLimit peak'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbTriangularDistribution class methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
distributionName
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^'Triangular distribution'! !

!DhbTriangularDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
fromHistogram: aHistogram
		"Create an instance of the receiver with parameters estimated from the
		  given histogram using best guesses. This method can be used to
		  find the initial values for a fit.
		 Default returns nil (must be implemented by subclass).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/3/99 "
	| b c|
	b := aHistogram standardDeviation * 1.73205080756888 "12 sqrt / 2".
	b = 0
		ifTrue: [ ^nil].
	c := aHistogram average.
	^self new: c from: ( c - b) to: ( c + b).! !

!DhbTriangularDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
new
		"Create an instance of the receiver with peak at 1/2 and limits 0 and 1.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^self new: (1 / 2) from: 0 to: 1! !

!DhbTriangularDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
new: aNumber1 from: aNumber2 to: aNumber3
		"Create an instance of the receiver with peak at aNumber1 and limits aNumber2 and aNumber3.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^super new initialize: aNumber1 from: aNumber2 to: aNumber3! !

!DhbTriangularDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
average
		"Answer the average of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^(lowLimit + peak + highLimit) / 3! !

!DhbTriangularDistribution methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
changeParametersBy: aVector
		"Modify the parameters of the receiver by aVector.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/3/99 "
	lowLimit := lowLimit + ( aVector at: 1).
	highLimit := highLimit + ( aVector at: 2).
	peak := peak + ( aVector at: 3).! !

!DhbTriangularDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
distributionValue: aNumber
		"Answers the probability of observing a random variable distributed according to
		 the receiver with a value lower than or equal to aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
		"changed last line so that 1 is returned, if aNumber >highLimit "
	| norm |
	^( aNumber between: lowLimit and: highLimit)
		ifTrue: [ aNumber < peak
						ifTrue: [ norm := ( highLimit - lowLimit) * ( peak - lowLimit).
									 ( aNumber - lowLimit) squared / norm
									]
						ifFalse:[ aNumber > peak
										ifTrue: [ norm := ( highLimit - lowLimit) * ( highLimit - peak).
									  				 1 - ( ( highLimit - aNumber) squared / norm)
													]
										ifFalse:[ ( peak - lowLimit) / ( highLimit - lowLimit)]
									]
				   ]
		ifFalse:[ aNumber >lowLimit ifTrue: [1] ifFalse: [0]]! !

!DhbTriangularDistribution methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialize: aNumber1 from: aNumber2 to: aNumber3
		"Private - Defines the parameters of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	( aNumber2 < aNumber3 and: [ aNumber1 between: aNumber2 and: aNumber3])
		ifFalse: [ self error: 'Illegal distribution parameters'].
	peak := aNumber1.
	lowLimit := aNumber2.
	highLimit := aNumber3.
	^self! !

!DhbTriangularDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
inverseAcceptanceAfterPeak: aNumber
		"Private - Compute inverse acceptance function in the region after the peak.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^ highLimit - ( ( ( 1 - aNumber) * ( highLimit - lowLimit) * ( highLimit - peak)) sqrt)! !

!DhbTriangularDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
inverseAcceptanceBeforePeak: aNumber
		"Private - Compute inverse acceptance function in the region before the peak.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^ ( aNumber * ( highLimit - lowLimit) * ( peak - lowLimit)) sqrt + lowLimit! !

!DhbTriangularDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
kurtosis
		"Answer the kurtosis of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^(-6/10)! !

!DhbTriangularDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
parameters
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^Array with: lowLimit with: highLimit with: peak! !

!DhbTriangularDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
privateInverseDistributionValue: aNumber
		"Private - Answer the number whose acceptance is aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^( peak - lowLimit) / ( highLimit - lowLimit) > aNumber
			ifTrue: [ self inverseAcceptanceBeforePeak: aNumber]
			ifFalse: [ self inverseAcceptanceAfterPeak: aNumber]! !

!DhbTriangularDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
skewness
		"Answer the skewness of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^(((lowLimit squared * lowLimit + ( peak squared * peak) + ( highLimit squared * highLimit) ) / 135)
	-(((lowLimit squared * peak) + (lowLimit squared * highLimit) + (peak squared * lowLimit) + (peak squared * highLimit) + (highLimit squared * lowLimit) + (highLimit squared * peak))/90)
	+( 2 * lowLimit * peak * highLimit / 45)) / ( self standardDeviation raisedToInteger: 3)! !

!DhbTriangularDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
value: aNumber
		"Answers the probability that a random variable distributed according to the receiver
		 gives a value between aNumber and aNumber + espilon (infinitesimal interval).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	| norm |
	^( aNumber between: lowLimit and: highLimit)
		ifTrue: [ aNumber < peak
						ifTrue: [ norm := ( highLimit - lowLimit) * ( peak - lowLimit).
									 2 * ( aNumber - lowLimit) / norm
									]
						ifFalse:[ aNumber > peak
										ifTrue: [ norm := ( highLimit - lowLimit) * ( highLimit - peak).
									  				 2 * ( highLimit - aNumber) / norm
													]
										ifFalse:[ 2 / ( highLimit - lowLimit)]
									]
				   ]
		ifFalse:[ 0]! !

!DhbTriangularDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
variance
		"Answer the variance of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^(lowLimit squared + peak squared + highLimit squared - ( lowLimit * peak) - ( lowLimit * highLimit) - ( peak * highLimit)) / 18! !

DhbProbabilityDensity subclass: #DhbUniformDistribution
	instanceVariableNames: 'lowLimit highLimit'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbUniformDistribution class methodsFor: 'public' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
distributionName
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^'Uniform distribution'! !

!DhbUniformDistribution class methodsFor: 'public' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
from: aNumber1 to: aNumber2
		"Create a new instance of the receiver with given limits.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^super new initialize: aNumber1 to: aNumber2! !

!DhbUniformDistribution class methodsFor: 'public' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
fromHistogram: aHistogram
		"Create an instance of the receiver with parameters estimated from the
		  given histogram using best guesses. This method can be used to
		  find the initial values for a fit.
		 Default returns nil (must be implemented by subclass).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/3/99 "
	| b c|
	b := aHistogram standardDeviation * 1.73205080756888 "12 sqrt / 2".
	b = 0
		ifTrue: [ ^nil].
	c := aHistogram average.
	^self from: ( c - b) to: ( c + b).! !

!DhbUniformDistribution class methodsFor: 'public' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
new
		"Create a new instance of the receiver with limits 0 and 1.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^self from: 0 to: 1! !

!DhbUniformDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
average
		"Answer the average of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^( highLimit + lowLimit) / 2! !

!DhbUniformDistribution methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
changeParametersBy: aVector
		"Modify the parameters of the receiver by aVector.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/3/99 "
	lowLimit := lowLimit + ( aVector at: 1).
	highLimit := highLimit + ( aVector at: 2).! !

!DhbUniformDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
distributionValue: aNumber
		"Answers the probability of observing a random variable distributed according to
		 the receiver with a value lower than or equal to aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	aNumber < lowLimit
		ifTrue: [ ^0].
	^aNumber < highLimit
		ifTrue: [ (aNumber - lowLimit) / ( highLimit - lowLimit)]
		ifFalse:[ 1]! !

!DhbUniformDistribution methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialize: aNumber1 to: aNumber2
		"Private - Defines the limits of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	aNumber1 < aNumber2
		ifFalse: [ self error: 'Illegal distribution parameters'].
	lowLimit := aNumber1.
	highLimit := aNumber2.
	^self! !

!DhbUniformDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
kurtosis
		"Answer the kurtosis of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^-12 / 10! !

!DhbUniformDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
parameters
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^Array with: lowLimit with: highLimit! !

!DhbUniformDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
privateInverseDistributionValue: aNumber
		"Private - Answer the number whose acceptance is aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^(highLimit - lowLimit) * aNumber + lowLimit! !

!DhbUniformDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
skewness
		"Answer the skewness of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 16/2/99 "
	^0! !

!DhbUniformDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
standardDeviation
		"Answer the standard deviation of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^( highLimit - lowLimit) / 3.46410161513775 "12 sqrt"! !

!DhbUniformDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
value: aNumber
		"Answers the probability that a random variable distributed according to the receiver
		 gives a value between aNumber and aNumber + espilon (infinitesimal interval).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^( aNumber between: lowLimit and: highLimit)
		ifTrue: [ 1/( highLimit - lowLimit)]
		ifFalse:[ 0]! !

!DhbUniformDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
variance
		"Answer the variance of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^( highLimit - lowLimit) squared / 12! !

DhbProbabilityDensity subclass: #DhbWeibullDistribution
	instanceVariableNames: 'alpha beta norm'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbWeibullDistribution class methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
distributionName
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^'Weibull distribution'! !

!DhbWeibullDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
fromHistogram: aHistogram
		"Create an instance of the receiver with parameters estimated from the
		  given histogram using best guesses. This method can be used to
		  find the initial values for a fit.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 3/3/99 "
	| average xMin xMax accMin accMax |
	aHistogram minimum < 0
		ifTrue: [ ^nil].
	average := aHistogram average.
	xMin := ( aHistogram minimum + average) / 2.
	accMin := ( aHistogram countsUpTo: xMin) / aHistogram totalCount.
	xMax := ( aHistogram maximum + average) / 2.
	accMax := ( aHistogram countsUpTo: xMax) / aHistogram totalCount.
	^[self solve: xMin acc: accMin upper: xMax acc: accMax]
			on: Error do: [ :signal | signal return: nil]! !

!DhbWeibullDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
new
		"Prevent using this message to create instances
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^self error: 'Illegal creation message for this class'! !

!DhbWeibullDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
shape: aNumber1 scale: aNumber2 
	"Create an instance of the receiver with given shape and scale parameters.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "

	^super new initialize: aNumber1 scale: aNumber2! !

!DhbWeibullDistribution class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
solve: lowX acc: lowAcc upper: highX acc: highAcc 
	"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/3/99 "

	| lowLnAcc highLnAcc deltaLnAcc lowLnX highLnX |
	lowLnAcc := (1 - lowAcc) ln negated ln.
	highLnAcc := (1 - highAcc) ln negated ln.
	deltaLnAcc := highLnAcc - lowLnAcc.
	lowLnX := lowX ln.
	highLnX := highX ln.
	^self shape: deltaLnAcc / (highLnX - lowLnX)
		scale: ((highLnAcc * lowLnX - (lowLnAcc * highLnX)) / deltaLnAcc) exp! !

!DhbWeibullDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
average
		"Answer the average of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^(1 / alpha) gamma * beta / alpha! !

!DhbWeibullDistribution methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
changeParametersBy: aVector
		"Modify the parameters of the receiver by aVector.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/3/99 "
	alpha := alpha + ( aVector at: 1).
	beta := beta + ( aVector at: 2).
	self computeNorm.! !

!DhbWeibullDistribution methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
computeNorm
		"Private - Compute the norm of the receiver because its parameters have changed.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/3/99 "
	norm := alpha/ ( beta raisedTo: alpha).! !

!DhbWeibullDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
distributionValue: aNumber
		"Answers the probability of observing a random variable distributed according to
		 the receiver with a value lower than or equal to aNumber.
		 Assumes that the value of the receiver is 0 for x < 0.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^aNumber > 0
		ifTrue: [ 1 - ( ( ( aNumber / beta) raisedTo: alpha) negated exp)]
		ifFalse:[ 0]! !

!DhbWeibullDistribution methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialize: aNumber1 scale: aNumber2
		"Private - Initialize the parameters of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	( aNumber1 > 0 and: [ aNumber2 > 0])
		ifFalse: [ self error: 'Illegal distribution parameters'].
	alpha := aNumber1.
	beta := aNumber2.
	self computeNorm.
	^self! !

!DhbWeibullDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
parameters
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/3/99 "
	^Array with: alpha with: beta! !

!DhbWeibullDistribution methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
privateInverseDistributionValue: aNumber
		"Private - Answer the number whose acceptance is aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^( (1 - aNumber) ln negated raisedTo: ( 1 / alpha)) * beta! !

!DhbWeibullDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
value: aNumber
		"Answers the probability that a random variable distributed according to the receiver
		 gives a value between aNumber and aNumber + espilon (infinitesimal interval).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^( ( aNumber / beta) raisedTo: alpha) negated exp * ( aNumber raisedTo: ( alpha - 1)) * norm! !

!DhbWeibullDistribution methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
variance
		"Answer the variance of the receiver.
		 NOTE: At least one of the methods variance or standardDeviation must be implemented by the subclass.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^( beta squared / alpha) * ( (2 / alpha) gamma * 2 - ( (1 / alpha ) gamma squared / alpha))! !

!Integer methodsFor: '*Math-DHB-Numerical' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
gamma
	self > 0
		ifFalse: [^ self error: 'Attempt to compute the Gamma function of a non-positive integer'].
	^ (self - 1) factorial.! !

!Integer methodsFor: '*Math-DHB-Numerical' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
random
		"Answer a random integer between 0 and the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^ DhbMitchellMooreGenerator new integerValue: self! !

!Collection methodsFor: '*Math-DHB-Numerical' stamp: 'WernerKassens 4/15/2014 09:56'!
asDhbVector
	"Convert the receiver to a vector. 
	(c) Copyrights Didier BESSET, 1999, all rights reserved. 
	Initial code: 11/2/99"

	| aVector index |
	aVector := DhbVector new: self size.
	index := 0.
	self do: [:each | aVector at: (index := index + 1) put: each].
	^aVector! !

DhbIterativeProcess subclass: #DhbClusterFinder
	instanceVariableNames: 'dataServer dataSetSize minimumRelativeClusterSize'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbClusterFinder class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
new: anInteger server: aClusterDataServer type: aClusterClass
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/3/99 "
	^super new initialize: anInteger server: aClusterDataServer type: aClusterClass! !

!DhbClusterFinder methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
accumulate: aVector
		"Private - Accumulate aVector into the nearest cluster.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/3/99 "
	( result at: ( self indexOfNearestCluster: aVector)) accumulate: aVector.! !

!DhbClusterFinder methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
clusters: aCollectionOfClusters
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/18/00 "
	result := aCollectionOfClusters.! !

!DhbClusterFinder methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
collectChangesAndResetClusters
	"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "

	| hasEmptyClusters changes |
	changes := 0.
	hasEmptyClusters := false.
	result do: 
			[:each | 
			changes := each changes + changes.
			( each isInsignificantIn: self)
				ifTrue: 
					[each centerOn: nil.
					hasEmptyClusters := true]
				ifFalse: [each reset].
			].
	hasEmptyClusters 
		ifTrue: [result := result reject: [:each | each isUndefined]].
	^changes / 2! !

!DhbClusterFinder methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
dataServer: aClusterDataServer
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/18/00 "
	dataServer := aClusterDataServer.! !

!DhbClusterFinder methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
evaluateIteration
		"Perform an accumulation of the data from the server.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/3/99 "
	dataServer reset.
	dataSetSize := 0.
	[ dataServer atEnd]
		whileFalse:[ self accumulate: dataServer next.
					 dataSetSize := dataSetSize + 1.
					].
	^self collectChangesAndResetClusters! !

!DhbClusterFinder methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
finalizeIterations
	"Close the data server.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/3/99 "

	dataServer close! !

!DhbClusterFinder methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
indexOfNearestCluster: aVector
		"Private - Answers the index of the cluster nearest to aVector.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/3/99 "
	| distance index |
	index := 1.
	distance := ( result at: 1) distanceTo: aVector.
	2 to: result size do:
		[ :n | | x |
		  x := ( result at: n) distanceTo: aVector.
		  x < distance
			ifTrue: [ distance := x.
					  index := n.
					].
		].
	^index! !

!DhbClusterFinder methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialize: anInteger server: aClusterDataServer type: aClusterClass
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/3/99 "
	self dataServer: aClusterDataServer.
	self clusters: ( (1 to: anInteger) collect: [ :n | aClusterClass new]).
	minimumRelativeClusterSize := 0.
	^self! !

!DhbClusterFinder methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initializeIterations
	"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/3/99 "

	dataServer open.
	result 
		do: [:each | each isUndefined ifTrue: [each centerOn: dataServer next]]! !

!DhbClusterFinder methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
minimumClusterSize
	^(minimumRelativeClusterSize * dataSetSize) rounded! !

!DhbClusterFinder methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
minimumRelativeClusterSize: aNumber
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	minimumRelativeClusterSize := aNumber max: 0.! !

!DhbClusterFinder methodsFor: 'display' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
printOn: aStream
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	aStream nextPutAll: 'Iterations: '.
	iterations printOn: aStream.
	result do: [ :each | aStream cr. each printOn: aStream].! !

DhbIterativeProcess subclass: #DhbFunctionalIterator
	instanceVariableNames: 'functionBlock'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbFunctionalIterator commentStamp: '<historical>' prior: 0!
A DhbFunctionalIterator is an abstract base class for IterativeProcesses operating on a function. In the sense of these classes, a function is a block or object responding to the value: selector. 

Subclasses will redefine (in addition to methods prescribed in IterativeProcess)
	computeInitialValues 

New instances may be created with the function: class method, or existing instances may be assigned a new function with setFunction:

All functions are required to be single argument blocks, so multivariable functions must be wrapped in a way that value: takes an array of values as input.

!

DhbFunctionalIterator subclass: #DhbBisectionZeroFinder
	instanceVariableNames: 'positiveX negativeX'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbBisectionZeroFinder methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
computeInitialValues
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 22/4/99 "
	positiveX isNil
		ifTrue: [ self error: 'No positive value supplied'].
	negativeX isNil
		ifTrue: [ self error: 'No negative value supplied'].! !

!DhbBisectionZeroFinder methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
evaluateIteration
		"Perform one step of bisection.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 22/4/99 "
	result := ( positiveX + negativeX) * 0.5.
	( functionBlock value: result) > 0
		ifTrue: [ positiveX := result]
		ifFalse:[ negativeX := result].
	^self relativePrecision: ( positiveX - negativeX) abs! !

!DhbBisectionZeroFinder methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
findNegativeXFrom: aNumber1 range: aNumber2
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 22/4/99 "
	| n |
	n := 0.
	[ negativeX := Number random * aNumber2 + aNumber1.
	  ( functionBlock value: negativeX) < 0
		] whileFalse: [ n := n + 0.1.
						n > maximumIterations
							ifTrue: [ self error: 'Unable to find a negative function value'].
					  ].! !

!DhbBisectionZeroFinder methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
findPositiveXFrom: aNumber1 range: aNumber2
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 22/4/99 "
	| n |
	n := 0.
	[ positiveX := Number random * aNumber2 + aNumber1.
	  ( functionBlock value: positiveX) > 0
		] whileFalse: [ n := n + 1.
						n > maximumIterations
							ifTrue: [ self error: 'Unable to find a positive function value'].
					  ].! !

!DhbBisectionZeroFinder methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
setNegativeX: aNumber
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 22/4/99 "
	( functionBlock value: aNumber) < 0
		ifFalse:[ self error: 'Function is not negative at x = ', aNumber printString].
	negativeX := aNumber.! !

!DhbBisectionZeroFinder methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
setPositiveX: aNumber
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 22/4/99 "
	( functionBlock value: aNumber) > 0
		ifFalse:[ self error: 'Function is not positive at x = ', aNumber printString].
	positiveX := aNumber.! !

DhbFunctionalIterator subclass: #DhbFunctionOptimizer
	instanceVariableNames: 'optimizingPointClass bestPoints'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbFunctionOptimizer class methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
defaultPrecision
		"Private - Answers the default precision for newly created instances.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "
	^super defaultPrecision * 100! !

!DhbFunctionOptimizer class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
forOptimizer: aFunctionOptimizer
		"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	^self new initializeForOptimizer: aFunctionOptimizer! !

!DhbFunctionOptimizer class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
maximizingFunction: aFunction
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	^super new initializeAsMaximizer; setFunction: aFunction! !

!DhbFunctionOptimizer class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
minimizingFunction: aFunction
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	^super new initializeAsMinimizer; setFunction: aFunction! !

!DhbFunctionOptimizer methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
addPointAt: aNumber 
	"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "

	bestPoints 
		add: (optimizingPointClass vector: aNumber function: functionBlock)! !

!DhbFunctionOptimizer methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
bestPoints
		"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	^bestPoints! !

!DhbFunctionOptimizer methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
finalizeIterations
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	result := bestPoints first position.! !

!DhbFunctionOptimizer methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
functionBlock
		"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	^functionBlock! !

!DhbFunctionOptimizer methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialValue: aVector
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 04-Jan-00 "
	result := aVector copy.! !

!DhbFunctionOptimizer methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialize
		"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	bestPoints := SortedCollection sortBlock: [ :a :b | a betterThan: b].
	^super initialize! !

!DhbFunctionOptimizer methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initializeAsMaximizer
		"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	optimizingPointClass := DhbMaximizingPoint.
	^self initialize! !

!DhbFunctionOptimizer methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initializeAsMinimizer
		"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	optimizingPointClass := DhbMinimizingPoint.
	^self! !

!DhbFunctionOptimizer methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initializeForOptimizer: aFunctionOptimizer
		"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	optimizingPointClass := aFunctionOptimizer pointClass.
	functionBlock := aFunctionOptimizer functionBlock.
	^self initialize! !

!DhbFunctionOptimizer methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
pointClass
		"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	^optimizingPointClass! !

!DhbFunctionOptimizer methodsFor: 'display' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
printOn: aStream
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	super printOn: aStream.
	bestPoints do: [ :each | aStream cr. each printOn: aStream].! !

DhbFunctionOptimizer subclass: #DhbGeneticOptimizer
	instanceVariableNames: 'chromosomeManager'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbGeneticOptimizer class methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
defaultMaximumIterations
		"Private - Answers the default maximum number of iterations for newly created instances.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "
	^500! !

!DhbGeneticOptimizer class methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
defaultPrecision
		"Private - Answers the default precision for newly created instances.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "
	^0! !

!DhbGeneticOptimizer methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
addPointAt: aNumber 
	"necessary since nan's and infinites can throw off calculations. and calculations become a tiny bit more efficient"
	| p |
 	p:=optimizingPointClass vector: aNumber function: functionBlock.
 	((p value isInfinite)or:[p value isNaN]) ifFalse: [bestPoints add: p].! !

!DhbGeneticOptimizer methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
chromosomeManager: aChromosomeManager
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 23-Feb-00 "
	chromosomeManager := aChromosomeManager.
	^self! !

!DhbGeneticOptimizer methodsFor: 'operation' stamp: 'wernerkassens 10/29/2014 15:10'!
collectPoints
	"Private - 
	(c) Copyrights Didier BESSET, 2000, all rights reserved. 
	Initial code: 23-Feb-00"

	| bestPoint |
	bestPoints isEmpty ifFalse: [bestPoint := bestPoints first].
	bestPoints removeAll.
	chromosomeManager population do: [:each | self addPointAt: each].
	bestPoint ifNotNil: [:b|bestPoints add: b].
	result := bestPoints first position! !

!DhbGeneticOptimizer methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
computePrecision
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	^1! !

!DhbGeneticOptimizer methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
evaluateIteration
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 23-Feb-00 "
	| randomScale |
	randomScale := self randomScale.
	randomScale at: randomScale size put: 1. "for floating point errors in #randomScale "
	chromosomeManager reset.
	[ chromosomeManager isFullyPopulated]
		whileFalse: [ self processRandomParents: randomScale].
	self collectPoints.
	^self computePrecision! !

!DhbGeneticOptimizer methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initializeIterations
	"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 23-Feb-00 "

	chromosomeManager randomizePopulation.
	self collectPoints! !

!DhbGeneticOptimizer methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
processRandomParents: aNumberArray
		"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 23-Feb-00 "
	chromosomeManager process: ( bestPoints at: ( self randomIndex: aNumberArray)) position
						and:  ( bestPoints at: ( self randomIndex: aNumberArray)) position.! !

!DhbGeneticOptimizer methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
randomIndex: aNumberArray
		"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 23-Feb-00 "
	| x n |
	x := Number random.
	n := 1.
	aNumberArray do: 
		[ :each |
		  x < each
			ifTrue: [ ^n].
		  n := n + 1.
		].
	 self error: 'should not be reached'.  "the original line was there because of a bug that should be taken care of now"! !

!DhbGeneticOptimizer methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
randomScale
		"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 23-Feb-00 "
		"changes take care of divide by zero error"
	| norm fBest fWorst answer|
	fBest := bestPoints first value.
	fWorst := bestPoints last value.
	norm :=fBest - fWorst.
	norm =0 ifTrue: [
	norm:= 1 / bestPoints size.
	answer := bestPoints collect: [:each| norm] ] ifFalse: [
		norm := 1 / norm.
		answer := bestPoints collect: [ :each | (each value - fWorst) * norm] ].
	norm := 1 / ( answer inject: 0 into: [ :sum :each | each + sum]).
	fBest := 0.
	^answer collect: [ :each | fBest := each * norm + fBest. fBest]

! !

DhbFunctionOptimizer subclass: #DhbHillClimbingOptimizer
	instanceVariableNames: 'unidimensionalFinder'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbHillClimbingOptimizer methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
computeInitialValues
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 05-Jan-00 "
	unidimensionalFinder := DhbOneVariableFunctionOptimizer forOptimizer: self.
	unidimensionalFinder desiredPrecision: desiredPrecision.
	bestPoints := ( 1 to: result size)
							collect: [ :n |
									   ( DhbVectorProjectedFunction function: functionBlock)
											direction: ( ( DhbVector new: result size)
															atAllPut: 0;
															at: n put: 1;
															yourself);
											yourself
									 ].! !

!DhbHillClimbingOptimizer methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
evaluateIteration
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "
	| oldResult |
	precision := 1.0.
	bestPoints inject: result
				 into: [ :prev :each | ( self minimizeDirection: each from: prev)].
	self shiftDirections.
	self minimizeDirection: bestPoints last.
	oldResult := result.
	result := bestPoints last origin.
	precision := 0.0.
	result with: oldResult do:
		[ :x0 :x1 |
		  precision := ( self precisionOf: (x0 - x1) abs relativeTo: x0 abs) max: precision.
		].
	^precision! !

!DhbHillClimbingOptimizer methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
finalizeIterations
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "! !

!DhbHillClimbingOptimizer methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
minimizeDirection: aVectorFunction 
	"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	^unidimensionalFinder
		reset;
		setFunction: aVectorFunction;
		addPointAt: 0;
		addPointAt: precision;
		addPointAt: precision negated;
		evaluate! !

!DhbHillClimbingOptimizer methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
minimizeDirection: aVectorFunction from: aVector 
	"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "

	^aVectorFunction
		origin: aVector;
		argumentWith: ( self minimizeDirection: aVectorFunction)! !

!DhbHillClimbingOptimizer methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
shiftDirections
		"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	| position delta firstDirection |
	firstDirection := bestPoints first direction.
	bestPoints inject: nil
					into: [ :prev :each |
							position isNil
								ifTrue: [ position := each origin]
								ifFalse:[ prev direction: each direction].
							each
							].
	position := bestPoints last origin - position.
	delta := position norm.
	delta > desiredPrecision
		ifTrue: [ bestPoints last direction: (position scaleBy: (1 / delta))]
		ifFalse:[ bestPoints last direction: firstDirection].! !

DhbFunctionOptimizer subclass: #DhbMultiVariableGeneralOptimizer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbMultiVariableGeneralOptimizer methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
computeInitialValues
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/29/00 "
	self range notNil
		ifTrue: [ self performGeneticOptimization].
	self performSimplexOptimization.! !

!DhbMultiVariableGeneralOptimizer methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
evaluateIteration
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "
	| optimizer |
	optimizer := DhbHillClimbingOptimizer forOptimizer: self.
	optimizer desiredPrecision: desiredPrecision;
			  maximumIterations: maximumIterations;
			  initialValue: result.
	result := optimizer evaluate.
	^optimizer precision! !

!DhbMultiVariableGeneralOptimizer methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
finalizeIterations
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "! !

!DhbMultiVariableGeneralOptimizer methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
origin
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/29/00 "
	^result! !

!DhbMultiVariableGeneralOptimizer methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
origin: anArrayOrVector
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/29/00 "
	result := anArrayOrVector.! !

!DhbMultiVariableGeneralOptimizer methodsFor: 'operation' stamp: 'WernerKassens 4/15/2014 09:57'!
performGeneticOptimization
		"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/29/00 "
	| optimizer manager |
	optimizer := DhbGeneticOptimizer forOptimizer: self.
	manager := DhbVectorChromosomeManager new: 100 mutation: 0.1 crossover: 0.1.
	manager origin: self origin asDhbVector; range: self range asDhbVector.
	optimizer chromosomeManager: manager.
	result := optimizer evaluate.! !

!DhbMultiVariableGeneralOptimizer methodsFor: 'operation' stamp: 'WernerKassens 4/15/2014 09:57'!
performSimplexOptimization
	"Private - 
	(c) Copyrights Didier BESSET, 2000, all rights reserved. 
	Initial code: 2/29/00"

	| optimizer |
	optimizer := DhbSimplexOptimizer forOptimizer: self.
	optimizer desiredPrecision: desiredPrecision sqrt; maximumIterations: maximumIterations; initialValue: result asDhbVector.
	result := optimizer evaluate! !

!DhbMultiVariableGeneralOptimizer methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
range
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/29/00 "
	^self bestPoints! !

!DhbMultiVariableGeneralOptimizer methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
range: anArrayOrVector
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/29/00 "
	bestPoints := anArrayOrVector.! !

DhbFunctionOptimizer subclass: #DhbOneVariableFunctionOptimizer
	instanceVariableNames: ''
	classVariableNames: 'GoldenSection'
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbOneVariableFunctionOptimizer class methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
defaultPrecision
		"Private - Answers the default precision for newly created instances.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "
	^DhbFloatingPointMachine new defaultNumericalPrecision * 10! !

!DhbOneVariableFunctionOptimizer class methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
goldenSection
	"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "

	GoldenSection isNil ifTrue: [GoldenSection := (3 - 5 sqrt) / 2].
	^GoldenSection! !

!DhbOneVariableFunctionOptimizer methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
computeInitialValues
	"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "

	[ bestPoints size  > 3] whileTrue: [ bestPoints removeLast].
	bestPoints size = 3
		ifTrue: [ self hasBracketingPoints
					ifFalse:[ bestPoints removeLast].
				].
	bestPoints size < 3
		ifTrue: [ ( DhbOptimizingBracketFinder forOptimizer: self) evaluate].! !

!DhbOneVariableFunctionOptimizer methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
computePrecision
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	^self precisionOf: ( ( bestPoints at: 2) position - ( bestPoints at: 3) position) abs
		   relativeTo: ( bestPoints at: 1) position abs! !

!DhbOneVariableFunctionOptimizer methodsFor: 'operation' stamp: 'WernerKassens 5/2/2014 13:46'!
evaluateIteration
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	self addPointAt: self nextXValue.
	bestPoints removeAt: self indexOfOuterPoint.
	^self computePrecision! !

!DhbOneVariableFunctionOptimizer methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
hasBracketingPoints
		"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	| x1 |
	x1 := ( bestPoints at: 1) position.
	^( ( bestPoints at: 2) position - x1) * (( bestPoints at: 3) position - x1) < 0! !

!DhbOneVariableFunctionOptimizer methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
indexOfOuterPoint
		"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	| inferior superior x |
	inferior := false.
	superior := false.
	x := bestPoints first position.
	2 to: 4 do:
		[ :n |
		  ( bestPoints at: n) position < x
				ifTrue: [ inferior
							ifTrue: [ ^n].
						  inferior := true.
						]
				ifFalse:[ superior
							ifTrue: [ ^n].
						  superior := true.
						].
		].! !

!DhbOneVariableFunctionOptimizer methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
nextXValue
	"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	| d3 d2 x1 |
	x1 := ( bestPoints at: 1) position.
	d2 := ( bestPoints at: 2) position - x1.
	d3 := ( bestPoints at: 3) position - x1.
	^( d3 abs > d2 abs ifTrue: [ d3]
					   ifFalse:[ d2]) * self class goldenSection + x1! !

!DhbOneVariableFunctionOptimizer methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
reset
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	[ bestPoints isEmpty] whileFalse: [ bestPoints removeLast].! !

DhbOneVariableFunctionOptimizer subclass: #DhbOptimizingBracketFinder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbOptimizingBracketFinder class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialPoints: aSortedCollection function: aFunction
		"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	^super new setInitialPoints: aSortedCollection; setFunction: aFunction! !

!DhbOptimizingBracketFinder methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
computeInitialValues
	"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "

	[bestPoints size < 2] whileTrue: [self addPointAt: Number random]! !

!DhbOptimizingBracketFinder methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
evaluateIteration
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	| x1 x2 |
	x1 := ( bestPoints at: 1) position.
	x2 := ( bestPoints at: 2) position.
	self addPointAt: ( x1 * 3 - ( x2 * 2)).
	precision := ( x2 - x1) * ( ( bestPoints at: 3) position - x1).
	self hasConverged
		ifFalse:[ bestPoints removeLast].
	^precision! !

!DhbOptimizingBracketFinder methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
finalizeIterations
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	result := bestPoints.! !

!DhbOptimizingBracketFinder methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initializeForOptimizer: aFunctionOptimizer
		"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	super initializeForOptimizer: aFunctionOptimizer.
	bestPoints := aFunctionOptimizer bestPoints.
	^self! !

!DhbOptimizingBracketFinder methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
setInitialPoints: aSortedCollection
		"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	bestPoints := aSortedCollection.! !

DhbFunctionOptimizer subclass: #DhbSimplexOptimizer
	instanceVariableNames: 'worstVector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbSimplexOptimizer class methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
defaultPrecision
		"Private - Answers the default precision for newly created instances.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "
	^DhbFloatingPointMachine new defaultNumericalPrecision * 1000! !

!DhbSimplexOptimizer methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
buildInitialSimplex
	"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "

	| projectedFunction finder partialResult |
	projectedFunction := DhbProjectedOneVariableFunction 
				function: functionBlock.
	finder := DhbOneVariableFunctionOptimizer forOptimizer: self.
	finder setFunction: projectedFunction.
	[bestPoints size < (result size + 1)] whileTrue: 
			[projectedFunction
				setArgument: result;
				bumpIndex.
			partialResult := finder
						reset;
						evaluate.
			bestPoints add: (optimizingPointClass 
						vector: (projectedFunction argumentWith: partialResult)
						function: functionBlock)]! !

!DhbSimplexOptimizer methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
computeInitialValues
	"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "

	bestPoints 
		add: (optimizingPointClass vector: result function: functionBlock).
	self buildInitialSimplex.
	worstVector := bestPoints removeLast position! !

!DhbSimplexOptimizer methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
computePrecision
		"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 1/16/00 "
	| functionValues bestFunctionValue |
	functionValues := bestPoints collect: [ :each | each value].
	bestFunctionValue := functionValues removeFirst.
	^functionValues inject: 0
					into: [ :max :each | ( self precisionOf: ( each - bestFunctionValue) abs relativeTo: bestFunctionValue abs) max: max]! !

!DhbSimplexOptimizer methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
contract
	"Private - Contract the Simplex around the best Vector.
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 1/16/00 "

	| bestVector oldVectors |
	bestVector := bestPoints first position.
	oldVectors := OrderedCollection with: worstVector.
	[bestPoints size > 1] whileTrue: [oldVectors add: bestPoints removeLast position].
	oldVectors do: [:each | self contract: each around: bestVector].
	worstVector := bestPoints removeLast position.
	^self computePrecision! !

!DhbSimplexOptimizer methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
contract: aVector around: bestVector 
	"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 1/16/00 "

	bestPoints 
		add: (optimizingPointClass vector: bestVector * 0.5 + (aVector * 0.5)
				function: functionBlock)! !

!DhbSimplexOptimizer methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
evaluateIteration
	"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "

	| centerOfGravity newPoint nextPoint |
	centerOfGravity := (bestPoints inject: ((worstVector copy)
						atAllPut: 0;
						yourself)
				into: [:sum :each | each position + sum]) * (1 / bestPoints size).
	newPoint := optimizingPointClass vector: 2 * centerOfGravity - worstVector
				function: functionBlock.
	(newPoint betterThan: bestPoints first) 
		ifTrue: 
			[nextPoint := optimizingPointClass 
						vector: newPoint position * 2 - centerOfGravity
						function: functionBlock.
			(nextPoint betterThan: newPoint) ifTrue: [newPoint := nextPoint]]
		ifFalse: 
			[newPoint := optimizingPointClass 
						vector: centerOfGravity * 0.666667 + (worstVector * 0.333333)
						function: functionBlock.
			(newPoint betterThan: bestPoints first) ifFalse: [^self contract]].
	worstVector := bestPoints removeLast position.
	bestPoints add: newPoint.
	result := bestPoints first position.
	^self computePrecision! !

!DhbSimplexOptimizer methodsFor: 'display' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
printOn: aStream
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	super printOn: aStream.
	aStream cr. 
	worstVector printOn: aStream.! !

!DhbFunctionalIterator class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
function: aBlock
		"Convenience method to create a instance with given function block.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 7/1/99 "
	^self new setFunction: aBlock; yourself! !

!DhbFunctionalIterator methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
computeInitialValues
	self subclassResponsibility! !

!DhbFunctionalIterator methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initializeIterations
	"If no initial value has been defined, take 0 as the starting point (for lack of anything better).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "

	functionBlock isNil ifTrue: [self error: 'No function supplied'].
	self computeInitialValues! !

!DhbFunctionalIterator methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
relativePrecision: aNumber
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 27/4/99 "
	^self precisionOf: aNumber relativeTo: result abs! !

!DhbFunctionalIterator methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
setFunction: aBlock
		"Defines the function for which zeroes will be found.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 7/1/99 "
	( aBlock respondsTo: #value:)
		ifFalse:[ self error: 'Function block must implement the method value:'].
	functionBlock := aBlock.! !

DhbFunctionalIterator subclass: #DhbNewtonZeroFinder
	instanceVariableNames: 'derivativeBlock'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

DhbNewtonZeroFinder subclass: #CdfNewtonZeroFinder
	instanceVariableNames: 'old'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!CdfNewtonZeroFinder commentStamp: 'WernerKassens 9/16/2015 17:19' prior: 0!
A CdfNewtonZeroFinder is a specialised version of DhbNewtonZeroFinder for DhbProbabilityDensity >>privateInverseDistributionValue:, that can deal with problems that can arise there (it doesnt diverge on a cdf)!

!CdfNewtonZeroFinder methodsFor: 'operation' stamp: 'WernerKassens 9/16/2015 17:38'!
computeInitialValues

	"super computeInitialValues." "the original testing there is skipped for speed reasons, not because of the error,that is produced here, which is completely eliminated"
	old:=result-1.0.! !

!CdfNewtonZeroFinder methodsFor: 'operation' stamp: 'WernerKassens 9/16/2015 17:34'!
evaluateIteration
		"fallback on bisection if necessary (no divideByZero anymore)"
	| delta new |
	delta :=( functionBlock value: result) / (derivativeBlock value: result).
	new := result - delta.
	old<result 
		ifTrue: [	new <= old ifTrue:[new:=old+result /2.0]]
		ifFalse:[	new>=old ifTrue: [new:=old+result /2.0]].
	[0.0 = (derivativeBlock value: new )] whileTrue: [new:=new +result /2.0].
	old:=result.
	result:=new.
	^self relativePrecision: delta abs! !

!CdfNewtonZeroFinder methodsFor: 'initialization' stamp: 'WernerKassens 9/16/2015 16:23'!
setDerivative: aBlock 
	"the original testing is skipped for speed reasons, not because of the error (which is largely eliminated anyway)."

	derivativeBlock := aBlock! !

!DhbNewtonZeroFinder class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
function: aBlock1 derivative: aBlock2
		"Convenience method to create a instance with given function block.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 7/1/99 "
	^(self new) setFunction: aBlock1; setDerivative: aBlock2; yourself! !

!DhbNewtonZeroFinder methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
computeInitialValues
		"Private - If no derivative has been defined, take an ad-hoc definition.
		 If no initial value has been defined, take 0 as the starting point (for lack of anything better).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "
	| n |
	result isNil
		ifTrue: [ result := 0].
	derivativeBlock isNil
		ifTrue: [ derivativeBlock := self defaultDerivativeBlock].
	n := 0.
	[ (derivativeBlock value: result) equalsTo: 0]
		whileTrue: [ n := n + 1.
					 n > maximumIterations
						ifTrue: [ self error: 'Function''s derivative seems to be zero everywhere'].
					 result := Number random + result].! !

!DhbNewtonZeroFinder methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
defaultDerivativeBlock
		"Private - Answers a block computing the function's derivative by approximation.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 7/1/99 "
	^[ :x | 5000 * ( ( functionBlock value: (x + 0.0001)) - ( functionBlock value: (x - 0.0001)))]! !

!DhbNewtonZeroFinder methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
evaluateIteration
		"Compute one step of Newton's zero finding method. Answers the estimated precision.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "
	| delta |
	delta := ( functionBlock value: result) / ( derivativeBlock value: result).
	result := result - delta.
	^self relativePrecision: delta abs! !

!DhbNewtonZeroFinder methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialValue: aNumber
		"Define the initial value for the iterations.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 7/1/99 "
	result := aNumber.! !

!DhbNewtonZeroFinder methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
setDerivative: aBlock 
	"Defines the derivative of the function for which zeroes will be found. 
	(c) Copyrights Didier BESSET, 1999, all rights reserved. 
	Initial code: 7/1/99"

	| x |
	(aBlock respondsTo: #value:)
		ifFalse: [self error: 'Derivative block must implement the method value:'].
	x := result isNil
				ifTrue: [Number random]
				ifFalse: [result + Number random].
	((aBlock value: x)
		relativelyEqualsTo: (self defaultDerivativeBlock value: x)
		upTo: 1.0e-4)
		ifFalse: [self error: 'Supplied derivative is not correct'].
	derivativeBlock := aBlock! !

!DhbNewtonZeroFinder methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
setFunction: aBlock
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 26/4/99 "
	super setFunction: aBlock.
	derivativeBlock := nil.! !

DhbFunctionalIterator subclass: #DhbTrapezeIntegrator
	instanceVariableNames: 'from to sum step'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbTrapezeIntegrator commentStamp: '<historical>' prior: 0!
A DhbTrapezeIntegrator is the base class for integration methods. You should not use this class for real applications, since much better results can be obtained from Romberg or Simpson integration.

!

DhbTrapezeIntegrator subclass: #DhbRombergIntegrator
	instanceVariableNames: 'order points interpolator'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbRombergIntegrator commentStamp: '<historical>' prior: 0!
A DhbRombergIntegrator implements the Romberg Method. This uses Richardson extrapolation on the Trapezoid method to generate a higher order estimate.

Evenly spaced samples are used. 

See also:
http://en.wikipedia.org/wiki/Romberg%27s_method

methods:

order: anInteger
	number of rows in the tableau.
	
!

!DhbRombergIntegrator class methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
defaultOrder
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 26/4/99 "
	^5! !

!DhbRombergIntegrator methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
computeInitialValues
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 26/4/99 "
	super computeInitialValues.
	points := OrderedCollection new: order.
	interpolator := DhbNevilleInterpolator points: points.
	points add: 1 @ sum.! !

!DhbRombergIntegrator methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
evaluateIteration
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 26/4/99 "
	| interpolation |
	points addLast: (points last x * 0.25) @ self higherOrderSum.
	points size < order
		ifTrue: [ ^1].
	interpolation := interpolator valueAndError: 0.
	points removeFirst.
	result := interpolation at: 1.
	^self relativePrecision: ( interpolation at: 2) abs! !

!DhbRombergIntegrator methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialize
		"Private - initialize the parameters of the receiver with default values.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "
	order := self class defaultOrder.
	^super initialize! !

!DhbRombergIntegrator methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
order: anInteger
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 26/4/99 "
	anInteger < 2
		ifTrue: [ self error: 'Order for Romberg integration must be larger than 1'].
	order := anInteger.! !

DhbTrapezeIntegrator subclass: #DhbSimpsonIntegrator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbSimpsonIntegrator methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
evaluateIteration
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 27/4/99 "
	| oldResult oldSum |
	iterations < 2
		ifTrue: [ self higherOrderSum.
				  ^1
				].
	oldResult := result.
	oldSum := sum.
	result := (self higherOrderSum * 4 - oldSum) / 3.
	^self relativePrecision: ( result - oldResult) abs! !

!DhbTrapezeIntegrator class methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
defaultMaximumIterations
		"Private - Answers the default maximum number of iterations for newly created instances.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "
	^13! !

!DhbTrapezeIntegrator class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
function: aBlock from: aNumber1 to: aNumber2
		"Create an new instance with given parameters.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 7/1/99 "
	^super new initialize: aBlock from: aNumber1 to: aNumber2! !

!DhbTrapezeIntegrator class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
new
		"Private - Block the constructor method for this class.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 7/1/99 "
	^self error: 'Method new:from:to: must be used'! !

!DhbTrapezeIntegrator methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
computeInitialValues
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 26/4/99 "
	step := to - from.
	sum := ( ( functionBlock value: from) + ( functionBlock value: to)) * step /2.
	result := sum.! !

!DhbTrapezeIntegrator methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
evaluateIteration
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 26/4/99 "
	| oldResult |
	oldResult := result.
	result := self higherOrderSum.
	^self relativePrecision: ( result - oldResult) abs! !

!DhbTrapezeIntegrator methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
from: aNumber1 to: aNumber2
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 26/4/99 "
	from := aNumber1.
	to := aNumber2.! !

!DhbTrapezeIntegrator methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
higherOrderSum
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 26/4/99 "
	| x newSum |
	x := step / 2 + from.
	newSum := 0.
	[ x < to ]
		whileTrue: [ newSum := ( functionBlock value: x) + newSum.
					 x := x + step.
				   ].
	sum := ( step * newSum + sum) / 2.
	step := step / 2.
	^sum! !

!DhbTrapezeIntegrator methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialize: aBlock from: aNumber1 to: aNumber2
		"Private - Initialize the parameters of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 7/1/99 "
	functionBlock := aBlock.
	self from: aNumber1 to: aNumber2.
	^self! !

DhbIterativeProcess subclass: #DhbIncompleteBetaFunctionFraction
	instanceVariableNames: 'x q1 q2 q3 numerator denominator alpha1 alpha2'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbIncompleteBetaFunctionFraction class methodsFor: 'creation' stamp: 'StephaneDucasse 7/14/2015 13:32'!
shape: aNumber1 shape: aNumber2
	"Create an instance of the receiver with given shape parameters."

	^ super new initialize: aNumber1 shape: aNumber2! !

!DhbIncompleteBetaFunctionFraction methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
evaluateIteration
	"Compute and add the next term of the fraction.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 10/3/99 "

	| m m2 temp |
	m := iterations + 1.
	m2 := m * 2.
	temp := m * (alpha2 - m) * x 
				/ ((q3 + m2) * (alpha1 + m2)).
	denominator := self limitedSmallValue: ( denominator * temp + 1).
	numerator := self limitedSmallValue: ( temp / numerator + 1).
	denominator := 1 / denominator.
	result := result * numerator * denominator.
	temp := (alpha1 + m) negated * (q1 + m) * x 
				/ ((q2 + m2) * (alpha1 + m2)).
	denominator := self limitedSmallValue: ( denominator * temp + 1).
	numerator := self limitedSmallValue: ( temp / numerator + 1).
	denominator := 1 / denominator.
	temp := numerator * denominator.
	result := result * temp.
	^(temp - 1) abs! !

!DhbIncompleteBetaFunctionFraction methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialize: aNumber1 shape: aNumber2 
	"Private - Initialize the parameters of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 10/3/99 "

	alpha1 := aNumber1.
	alpha2 := aNumber2.
	q1 := alpha1 + alpha2.
	q2 := alpha1 + 1.
	q3 := alpha1 - 1.
	^self! !

!DhbIncompleteBetaFunctionFraction methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initializeIterations
	"Initialize the iterations (subclasses must write their own method and call this one last).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 10/3/99 "

	numerator := 1.
	denominator := 1 / (self limitedSmallValue: 1 - (q1 * x / q2)).
	result := denominator! !

!DhbIncompleteBetaFunctionFraction methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
setArgument: aNumber
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 10/3/99 "
	x := aNumber.! !

DhbIterativeProcess subclass: #DhbJacobiTransformation
	instanceVariableNames: 'lowerRows transform'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbJacobiTransformation commentStamp: 'StephaneDucasse 7/5/2015 21:22' prior: 0!
[[[ 
| m jacobi eigenvalues eigenvectors |
m := DhbSymmetricMatrix rows: #((84 -79 58 55)
                                 (-79 84 -55 -58)
                                 (58 -55 84 79)
                                 (55 -58 79 84)).
jacobi := DhbJacobiTransformation matrix: m.
eigenvalues := jacobi evaluate.
eigenvectors := jacobi transform columnsCollect: [ :each | each].
]]]!

!DhbJacobiTransformation class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
matrix: aSymmetricMatrix
		"
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 1/6/99 "
	^super new initialize: aSymmetricMatrix! !

!DhbJacobiTransformation class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
new
		"Prevent using this message to create instances.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 1/6/99 "
	^self error: 'Illegal creation message for this class'! !

!DhbJacobiTransformation methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
evaluateIteration
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 1/6/99 "
	| indices |
	indices := self largestOffDiagonalIndices.
	self transformAt: ( indices at: 1) and: ( indices at: 2).
	^precision! !

!DhbJacobiTransformation methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
exchangeAt: anInteger
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 1/6/99 "
	| temp n |
	n := anInteger + 1.
	temp := result at: n.
	result at: n put: ( result at: anInteger).
	result at: anInteger put: temp.
	transform do:
		[ :each |
		  temp := each at: n.
		  each at: n put: ( each at: anInteger).
		  each at: anInteger put: temp.
		].! !

!DhbJacobiTransformation methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
finalizeIterations
	"Transfer the eigenValues into a vector and set this as the result.
		 eigen values and transform matrix are sorted using a bubble sort.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 1/6/99 "

	| n |
	n := 0.
	result := lowerRows collect: 
					[:each | 
					n := n + 1.
					each at: n].
	self sortEigenValues! !

!DhbJacobiTransformation methodsFor: 'initialization' stamp: 'wernerkassens 10/29/2014 14:57'!
initialize: aSymmetricMatrix
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 1/6/99 "
	| n |
	n := aSymmetricMatrix numberOfRows.
	lowerRows := Array new: n.
	transform := Array new: n.
	1 to: n do:
		[ :k |
		  lowerRows at: k put: ( ( aSymmetricMatrix rowAt: k) copyFrom: 1 to: k).
		  transform at: k put: ( ( Array new: n) atAllPut: 0; at: k put: 1; yourself).
		].
	^self! !

!DhbJacobiTransformation methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
largestOffDiagonalIndices
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 1/6/99 "
	| n m abs |
	n := 2.
	m := 1.
	precision := ( ( lowerRows at: n) at: m) abs.
	1 to: lowerRows size do:
		[ :i |
		  1 to: ( i - 1) do:
			[ :j |
			  abs := ( ( lowerRows at: i) at: j) abs.
			  abs > precision
				ifTrue: [ n := i.
						  m := j.
						  precision := abs.
						]. 
			].
		].
	^Array with: m with: n! !

!DhbJacobiTransformation methodsFor: 'display' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
printOn: aStream
		"Append to the argument aStream, a sequence of characters that describes the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 1/6/99 "
	| first |
	first := true.
	lowerRows do: 
		[ :each |
		  first ifTrue: [ first := false]
				 ifFalse:[ aStream cr].
		  each printOn: aStream.
		].! !

!DhbJacobiTransformation methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
sortEigenValues
		"Private - Use a bubble sort.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 1/6/99 "
	| n bound m |
	n := lowerRows size.
	bound := n.
	[ bound = 0 ]
		whileFalse: [ m := 0.
					  1 to: bound - 1 do:
						[ :j |
						  ( result at: j) abs > ( result at: j + 1) abs
							ifFalse:[ self exchangeAt: j.
									  m := j.
									].
						].
					  bound := m.
					].! !

!DhbJacobiTransformation methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
transform
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 1/6/99 "
	^DhbMatrix rows: transform! !

!DhbJacobiTransformation methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
transformAt: anInteger1 and: anInteger2
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 1/6/99 "
	| d t s c tau apq app aqq arp arq |
	apq := ( lowerRows at: anInteger2) at: anInteger1.
	apq = 0
		ifTrue: [ ^nil].
	app := ( lowerRows at: anInteger1) at: anInteger1.
	aqq := ( lowerRows at: anInteger2) at: anInteger2.
	d := aqq - app.
	arp := d * 0.5 / apq.
	t := arp > 0 ifTrue: [ 1 / ( ( arp squared + 1) sqrt + arp)]
				 ifFalse:[ 1 / ( arp - ( arp squared + 1) sqrt)].
	c := 1 / ( t squared + 1) sqrt.
	s := t * c.
	tau := s / ( 1 + c).
	1 to: ( anInteger1 - 1)
		do: [ :r |
			  arp := ( lowerRows at: anInteger1) at: r.
			  arq := ( lowerRows at: anInteger2) at: r.
			  ( lowerRows at: anInteger1) at: r put: ( arp - ( s * (tau * arp + arq))).
			  ( lowerRows at: anInteger2) at: r put: ( arq + ( s * (arp - (tau * arq)))).
			].
	( anInteger1 + 1) to: ( anInteger2 - 1)
		do: [ :r |
			  arp := ( lowerRows at: r) at: anInteger1.
			  arq := ( lowerRows at: anInteger2) at: r.
			  ( lowerRows at: r) at: anInteger1 put: ( arp - ( s * (tau * arp + arq))).
			  ( lowerRows at: anInteger2) at: r put: ( arq + ( s * (arp - (tau * arq)))).
			].
	( anInteger2 + 1) to: lowerRows size
		do: [ :r |
			  arp := ( lowerRows at: r) at: anInteger1.
			  arq := ( lowerRows at: r) at: anInteger2.
			  ( lowerRows at: r) at: anInteger1 put: ( arp - ( s * (tau * arp + arq))).
			  ( lowerRows at: r) at: anInteger2 put: ( arq + ( s * (arp - (tau * arq)))).
			].
	1 to: lowerRows size
		do: [ :r |
			  arp := ( transform at: r) at: anInteger1.
			  arq := ( transform at: r) at: anInteger2.
			  ( transform at: r) at: anInteger1 put: ( arp - ( s * (tau * arp + arq))).
			  ( transform at: r) at: anInteger2 put: ( arq + ( s * (arp - (tau * arq)))).
			].
	( lowerRows at: anInteger1) at: anInteger1 put: ( app - (t * apq)).
	( lowerRows at: anInteger2) at: anInteger2 put: ( aqq + (t * apq)).
	( lowerRows at: anInteger2) at: anInteger1 put: 0.! !

DhbIterativeProcess subclass: #DhbLargestEigenValueFinder
	instanceVariableNames: 'matrix eigenvector transposeEigenvector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical-Math-Eigenvalues'!

!DhbLargestEigenValueFinder class methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
defaultMaximumIterations
		"Private - Answers the default maximum number of iterations for newly created instances.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "
	^100! !

!DhbLargestEigenValueFinder class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
matrix: aMatrix
		"Create a new instance of the receiver for a given matrix and default precision.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^( self new) initialize: aMatrix; yourself! !

!DhbLargestEigenValueFinder class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
matrix: aMatrix precision: aNumber
		"Create a new instance of the receiver for a given matrix and desired precision.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^( self new) initialize: aMatrix; desiredPrecision: aNumber; yourself! !

!DhbLargestEigenValueFinder methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
eigenvalue
	"Answer the eigen value found by the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "

	^result! !

!DhbLargestEigenValueFinder methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
eigenvector
	"Answer the normalized eigen vector found by the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "

	^eigenvector * (1 / eigenvector norm)! !

!DhbLargestEigenValueFinder methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
evaluateIteration
	"Iterate the product of the matrix of the eigen vector and the transpose.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "

	| oldEigenvalue |
	oldEigenvalue := result.
	transposeEigenvector := transposeEigenvector * matrix.
	transposeEigenvector := transposeEigenvector 
				* (1 / (transposeEigenvector at: 1)).
	eigenvector := matrix * eigenvector.
	result := eigenvector at: 1.
	eigenvector := eigenvector * (1 / result).
	^oldEigenvalue isNil 
		ifTrue: [2 * desiredPrecision]
		ifFalse: [(result - oldEigenvalue) abs]! !

!DhbLargestEigenValueFinder methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialize: aMatrix
		"Defines the matrix for the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	matrix := aMatrix.! !

!DhbLargestEigenValueFinder methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initializeIterations
	"Initialize the iterations (subclasses must write their own method and call this one last).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "

	eigenvector := DhbVector new: matrix numberOfRows.
	eigenvector atAllPut: 1.0.
	transposeEigenvector := DhbVector new: eigenvector size.
	transposeEigenvector atAllPut: 1.0! !

!DhbLargestEigenValueFinder methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
nextLargestEigenValueFinder
	"Return an eigen value finder for the same eigen values of the receiver except the one found.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "

	| norm |
	norm := 1 / (eigenvector * transposeEigenvector).
	^self class 
		matrix: matrix * ((DhbSymmetricMatrix identity: eigenvector size) 
						- (eigenvector * norm tensorProduct: transposeEigenvector))
		precision: desiredPrecision! !

DhbIterativeProcess subclass: #DhbLeastSquareFit
	instanceVariableNames: 'dataHolder errorMatrix chiSquare equations constants degreeOfFreedom'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbLeastSquareFit class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
histogram: aHistogram distributionClass: aProbabilityDensityFunctionClass 
	"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/3/99 "

	^self points: aHistogram
		function: (DhbScaledProbabilityDensityFunction histogram: aHistogram
				distributionClass: aProbabilityDensityFunctionClass)! !

!DhbLeastSquareFit class methodsFor: 'creation' stamp: 'wernerkassens 10/29/2014 15:34'!
points: aDataHolder function: aParametricFunction 
	"(c) Copyrights Didier BESSET, 1999, all rights reserved. 
	Initial code: 11/3/99"

	^aParametricFunction isNil
		ifTrue: [nil]
		ifFalse: [self new initialize: aDataHolder data: aParametricFunction]! !

!DhbLeastSquareFit methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
accumulate: aWeightedPoint
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/3/99 "
	| f g |
	f := result valueAndGradient: aWeightedPoint xValue.
	g := f last.
	f := f first.
	constants accumulate: g * ( ( aWeightedPoint yValue - f) * aWeightedPoint weight).
	1 to: g size do:
		[ :k |
		  ( equations at: k) accumulate: g * ( ( g at: k) * aWeightedPoint weight).
		].! !

!DhbLeastSquareFit methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
accumulateEquationSystem
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/3/99 "
	dataHolder pointsAndErrorsDo: [ :each | self accumulate: each].! !

!DhbLeastSquareFit methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
chiSquare
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/4/99 "
	chiSquare isNil
		ifTrue: [ self computeChiSquare].
	^chiSquare! !

!DhbLeastSquareFit methodsFor: 'operation' stamp: 'WernerKassens 4/6/2014 22:36'!
computeChanges
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/3/99 "
	errorMatrix := DhbLUPDecomposition direct: equations.
	^(errorMatrix solve: constants)ifNil:[SingularMatrixError new signal:'singular Errormatrix']! !

!DhbLeastSquareFit methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
computeChiSquare
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/4/99 "
	chiSquare := 0.
	degreeOfFreedom := self numberOfFreeParameters negated.
	dataHolder pointsAndErrorsDo:
		[ :each |
		  chiSquare := ( each chi2Contribution: result) + chiSquare.
		  degreeOfFreedom := degreeOfFreedom + 1.
		].! !

!DhbLeastSquareFit methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
computeEquationSystem
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/3/99 "
	constants atAllPut: 0.
	equations do: [ :each | each atAllPut: 0].
	self accumulateEquationSystem.! !

!DhbLeastSquareFit methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
confidenceLevel
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/6/00 "
	^( DhbChiSquareDistribution degreeOfFreedom: self degreeOfFreedom) confidenceLevel: self chiSquare! !

!DhbLeastSquareFit methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
degreeOfFreedom
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/4/99 "
	degreeOfFreedom isNil
		ifTrue: [ self computeChiSquare].
	^degreeOfFreedom! !

!DhbLeastSquareFit methodsFor: 'information' stamp: 'WernerKassens 5/2/2014 08:41'!
errorMatrix
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 14/4/99 "
	errorMatrix inverseMatrixComponents 
		ifNil: [ SingularMatrixError new signal:'singular Errormatrix' ] 
		ifNotNilDo: [ :i| ^DhbSymmetricMatrix rows: i ].
! !

!DhbLeastSquareFit methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
evaluateIteration
		"Dummy method (must be implemented by subclass).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "
	| changes maxChange|
	self computeEquationSystem.
	changes := self computeChanges.
	result changeParametersBy: changes.
	maxChange := 0.
	result parameters with: changes do: 
		[ :r :d | maxChange := ( d / r) abs max: maxChange].
	^maxChange! !

!DhbLeastSquareFit methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
finalizeIterations
	"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/3/99 "

	equations := nil.
	constants := nil.
	degreeOfFreedom := nil.
	chiSquare := nil! !

!DhbLeastSquareFit methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
fitType
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/4/99 "
	^'Least square fit'! !

!DhbLeastSquareFit methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialize: aDataHolder data: aParametricFunction
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/3/99 "
	dataHolder := aDataHolder.
	result := aParametricFunction.
	^self! !

!DhbLeastSquareFit methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initializeIterations
	"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "

	| n |
	n := self numberOfParameters.
	constants := (DhbVector new: n)
				atAllPut: 0;
				yourself.
	equations := (1 to: n) collect: 
					[:k | 
					(DhbVector new: n)
						atAllPut: 0;
						yourself]! !

!DhbLeastSquareFit methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
numberOfFreeParameters
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/4/99 "
	^self numberOfParameters! !

!DhbLeastSquareFit methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
numberOfParameters
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/3/99 "
	^result parameters size! !

!DhbLeastSquareFit methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
value: aNumber
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/8/00 "
	^result value: aNumber! !

!DhbLeastSquareFit methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
valueAndError: aNumber
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/8/00 "
	| valueGradient |
	valueGradient := result valueAndGradient: aNumber.
	^Array with: valueGradient first
		   with: ( valueGradient last * ( self errorMatrix * valueGradient last)) sqrt! !

DhbLeastSquareFit subclass: #DhbMaximumLikelihoodHistogramFit
	instanceVariableNames: 'count countVariance'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbMaximumLikelihoodHistogramFit methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
accumulate: aWeightedPoint
		"Private - 
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/3/99 "
	| f g temp inverseProbability|
	f := result valueAndGradient: aWeightedPoint xValue.
	g := f last copyFrom: 1 to: ( f last size - 1).
	f := f first.
	f = 0 ifTrue: [ ^nil].
	inverseProbability := 1 / f.
	temp := aWeightedPoint yValue * inverseProbability.
	constants accumulate: g * temp.
	temp := temp * inverseProbability.
	1 to: g size do:
		[ :k |
		  ( equations at: k) accumulate: g * ( ( g at: k) * temp).
		].! !

!DhbMaximumLikelihoodHistogramFit methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
computeChanges
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/3/99 "
	^super computeChanges copyWith: 0! !

!DhbMaximumLikelihoodHistogramFit methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
computeNormalization
	"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/3/99 "

	| numerator denominator temp |
	numerator := 0.
	denominator := 0.
	dataHolder pointsAndErrorsDo: 
			[:each | 
			temp := result value: each xValue.
			temp = 0 
				ifFalse: 
					[numerator := numerator + (each yValue squared / temp).
					denominator := denominator + temp]].
	count := ( numerator / denominator) sqrt.
	countVariance := numerator / ( 4 * count).! !

!DhbMaximumLikelihoodHistogramFit methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
finalizeIterations
	"Compute the normalization factor.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/3/99 "

	self computeNormalization.
	result setCount: count.
	super finalizeIterations! !

!DhbMaximumLikelihoodHistogramFit methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
fitType
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/4/99 "
	^'Maximum likelihood fit'! !

!DhbMaximumLikelihoodHistogramFit methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initializeIterations
	"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/3/99 "

	result setCount: 1.
	count := dataHolder totalCount.
	super initializeIterations! !

!DhbMaximumLikelihoodHistogramFit methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
normalization
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/12/00 "
	^count! !

!DhbMaximumLikelihoodHistogramFit methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
normalizationError
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/12/00 "
	^countVariance sqrt! !

!DhbMaximumLikelihoodHistogramFit methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
numberOfFreeParameters
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/4/99 "
	^super numberOfParameters! !

!DhbMaximumLikelihoodHistogramFit methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
numberOfParameters
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/3/99 "
	^super numberOfParameters - 1! !

!DhbMaximumLikelihoodHistogramFit methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
valueAndError: aNumber 
	"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/8/00 "

	| valueGradient gradient gVar |
	valueGradient := result valueAndGradient: aNumber.
	gradient := valueGradient last copyFrom: 1 to: valueGradient last size - 1.
	gVar := gradient * (self errorMatrix * gradient) / count.
	^Array with: valueGradient first
		with: ((valueGradient first / count) squared * countVariance + gVar) sqrt! !

ArithmeticError subclass: #SingularMatrixError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical-Math-Matrix'!

!SingularMatrixError commentStamp: 'WernerKassens 4/5/2014 11:53' prior: 0!
some calculations dont work with singular matrices and result eg with errors like 'receiver of last is nil', deep down in the calculation, which is quite ununderstandable. otoh these errors can occasionally be treated by using another calculation, hence i introduced this error.
!

!SingularMatrixError methodsFor: 'accessing' stamp: 'WernerKassens 4/5/2014 11:35'!
messageText
	"Overwritten to initialize the message text to a standard text if it has not yet been set"
	^ messageText ifNil: [ messageText := self standardMessageText ]! !

!SingularMatrixError methodsFor: 'printing' stamp: 'WernerKassens 4/6/2014 11:16'!
standardMessageText
	^ String streamContents: [ :stream |
		stream << (self signalerContext  asString findTokens: '>')last.
		stream << ' encountered a singular matrix']! !

Object subclass: #DhbAbstractDataServer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbAbstractDataServer methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
atEnd
		"Answers true if there is no more data element.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/3/99 "
	self subclassResponsibility! !

!DhbAbstractDataServer methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
close
		"Close the data stream (must be implemented by subclass).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/3/99 "! !

!DhbAbstractDataServer methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
next
		"Answers the next element on the stream.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/3/99 "
	self subclassResponsibility! !

!DhbAbstractDataServer methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
open
		"Open the data stream (must be implemented by subclass).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/3/99 "
	self subclassResponsibility! !

!DhbAbstractDataServer methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
reset
		"Reset the position of the data stream to the beginning.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/3/99 "
	self subclassResponsibility! !

DhbAbstractDataServer subclass: #DhbMemoryBasedDataServer
	instanceVariableNames: 'data position'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbMemoryBasedDataServer methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
atEnd
		"Answers true if there is no more data element.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/3/99 "
	^data size < position! !

!DhbMemoryBasedDataServer methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
data: anOrderedCollection
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	data := anOrderedCollection.
	self reset.! !

!DhbMemoryBasedDataServer methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
dimension
		"Answers the dimension of the vectors catered by the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/3/99 "
	^data first size! !

!DhbMemoryBasedDataServer methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
next
		"Answers the next element on the stream.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/3/99 "
	| answer |
	answer := data at: position.
	position := position + 1.
	^answer! !

!DhbMemoryBasedDataServer methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
open
		"Open the data stream (must be implemented by subclass).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/3/99 "
	self reset! !

!DhbMemoryBasedDataServer methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
reset
		"Reset the position of the data stream to the beginning.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/3/99 "
	position := 1.! !

Object subclass: #DhbChromosomeManager
	instanceVariableNames: 'population populationSize rateOfMutation rateOfCrossover'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbChromosomeManager class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
new: anInteger mutation: aNumber1 crossover: aNumber2
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 23-Feb-00 "
	^self new populationSize: anInteger; rateOfMutation: aNumber1; rateOfCrossover: aNumber2; yourself! !

!DhbChromosomeManager methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
clone: aChromosome
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 23-Feb-00 "
	^aChromosome copy! !

!DhbChromosomeManager methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
crossover: aChromosome1 and: aChromosome2
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 23-Feb-00 "
	^self subclassResponsibility! !

!DhbChromosomeManager methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
isFullyPopulated
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 23-Feb-00 "
	^population size >= populationSize! !

!DhbChromosomeManager methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
mutate: aChromosome
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 23-Feb-00 "
	^self subclassResponsibility! !

!DhbChromosomeManager methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
population
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 23-Feb-00 "
	^population! !

!DhbChromosomeManager methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
populationSize: anInteger
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 23-Feb-00 "
	populationSize := anInteger.! !

!DhbChromosomeManager methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
process: aChromosome1 and: aChromosome2 
	"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 23-Feb-00 "

	| roll |
	roll := Number random.
	roll < rateOfCrossover 
		ifTrue: [population addAll: (self crossover: aChromosome1 and: aChromosome2)]
		ifFalse: 
			[roll < (rateOfCrossover + rateOfMutation) 
				ifTrue: 
					[population
						add: (self mutate: aChromosome1);
						add: (self mutate: aChromosome2)]
				ifFalse: 
					[population
						add: (self clone: aChromosome1);
						add: (self clone: aChromosome2)]]! !

!DhbChromosomeManager methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
randomChromosome
	self subclassResponsibility! !

!DhbChromosomeManager methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
randomizePopulation
	"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 23-Feb-00 "

	self reset.
	[self isFullyPopulated] 
		whileFalse: [population add: self randomChromosome]! !

!DhbChromosomeManager methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
rateOfCrossover: aNumber 
	"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 23-Feb-00 "

	(aNumber between: 0 and: 1) 
		ifFalse: [self error: 'Illegal rate of cross-over'].
	rateOfCrossover := aNumber! !

!DhbChromosomeManager methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
rateOfMutation: aNumber 
	"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 23-Feb-00 "

	(aNumber between: 0 and: 1) 
		ifFalse: [self error: 'Illegal rate of mutation'].
	rateOfMutation := aNumber! !

!DhbChromosomeManager methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
reset
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 23-Feb-00 "
	population := OrderedCollection new: populationSize.! !

DhbChromosomeManager subclass: #DhbVectorChromosomeManager
	instanceVariableNames: 'origin range'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbVectorChromosomeManager methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
crossover: aChromosome1 and: aChromosome2
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 23-Feb-00 "
	| index new1 new2|
	index := ( aChromosome1 size - 1) random + 2.
	new1 := self clone: aChromosome1.
	new1 replaceFrom: index to: new1 size with: aChromosome2 startingAt: index.
	new2 := self clone: aChromosome2.
	new2 replaceFrom: index to: new2 size with: aChromosome1 startingAt: index.
	^Array with: new1 with: new2! !

!DhbVectorChromosomeManager methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
mutate: aVector
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 23-Feb-00 "
	| index |
	index := aVector size random + 1.
	^( aVector copy)
			at: index put: ( self randomComponent: index);
			yourself! !

!DhbVectorChromosomeManager methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
origin: aVector
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 23-Feb-00 "
	origin := aVector.! !

!DhbVectorChromosomeManager methodsFor: 'creation' stamp: 'WernerKassens 4/15/2014 09:56'!
randomChromosome
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 23-Feb-00 "
	^( ( 1 to: origin size) collect: [ :n | self randomComponent: n]) asDhbVector! !

!DhbVectorChromosomeManager methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
randomComponent: anInteger
		"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 23-Feb-00 "
	^( range at: anInteger) asFloat random + ( origin at: anInteger)! !

!DhbVectorChromosomeManager methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
range: aVector
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 23-Feb-00 "
	range := aVector.! !

Object subclass: #DhbCluster
	instanceVariableNames: 'accumulator previousSampleSize'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbCluster class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
new
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	^super new initialize! !

!DhbCluster methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
accumulate: aVector
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	accumulator accumulate: aVector.! !

!DhbCluster methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
centerOn: aVector
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	self subclassResponsibility! !

!DhbCluster methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
changes
	"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "

	^(self sampleSize - previousSampleSize) abs! !

!DhbCluster methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
collectAccumulatorResults
	self subclassResponsibility! !

!DhbCluster methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
distanceTo: aVector
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	^self subclassResponsibility! !

!DhbCluster methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialize
	"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "

	previousSampleSize := 0.
	^self! !

!DhbCluster methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
isInsignificantIn: aClusterFinder
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/17/00 "
	^self sampleSize <= aClusterFinder minimumClusterSize! !

!DhbCluster methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
isUndefined
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	^self subclassResponsibility! !

!DhbCluster methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
reset
	"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "

	previousSampleSize := self sampleSize.
	self collectAccumulatorResults.
	accumulator reset! !

!DhbCluster methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
sampleSize
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	^accumulator count! !

DhbCluster subclass: #DhbCovarianceCluster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbCovarianceCluster methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
centerOn: aVector 
	"(c) Copyrights Didier BESSET, 2000, all rights reserved. 
	Initial code: 2/16/00"

	accumulator := aVector isNil
				ifTrue: [nil]
				ifFalse: [DhbMahalanobisCenter onVector: aVector]! !

!DhbCovarianceCluster methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
collectAccumulatorResults
		"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	accumulator computeParameters.! !

!DhbCovarianceCluster methodsFor: 'display' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
distanceTo: aVector
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	^accumulator distanceTo: aVector! !

!DhbCovarianceCluster methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
isUndefined
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	^accumulator isNil! !

!DhbCovarianceCluster methodsFor: 'display' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
printOn: aStream
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	accumulator printOn: aStream.! !

DhbCluster subclass: #DhbEuclideanCluster
	instanceVariableNames: 'center'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbEuclideanCluster methodsFor: 'initialization' stamp: 'SergeStinckwich 1/25/2013 13:57'!
centerOn: aVector
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
		"minimal change so that it works in pharo 1.2.1" 
	center := aVector.
	accumulator := DhbVectorAccumulator new: (aVector ifNil:[0]ifNotNil:[aVector size]).! !

!DhbEuclideanCluster methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
collectAccumulatorResults
		"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	center := accumulator average copy.! !

!DhbEuclideanCluster methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
distanceTo: aVector
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	^( aVector - center) norm! !

!DhbEuclideanCluster methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
isUndefined
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	^center isNil! !

!DhbEuclideanCluster methodsFor: 'display' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
printOn: aStream
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	accumulator count printOn: aStream.
	aStream nextPutAll: ': '.
	center printOn: aStream.! !

Object subclass: #DhbCongruentialRandomNumberGenerator
	instanceVariableNames: 'constant modulus multiplicator seed'
	classVariableNames: 'UniqueInstance'
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbCongruentialRandomNumberGenerator class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
constant: aNumber1 multiplicator: aNumber2 modulus: aNumber3 
	"Create a new instance of the receiver with given constants.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "

	^super new 
		initialize: aNumber1
		multiplicator: aNumber2
		modulus: aNumber3! !

!DhbCongruentialRandomNumberGenerator class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
new
		"Create a new instance of the receiver with D. Knuth's constants.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	UniqueInstance isNil
		ifTrue: [ UniqueInstance := super new initialize.
					 UniqueInstance setSeed: 1.
				   ].
	^UniqueInstance! !

!DhbCongruentialRandomNumberGenerator class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
seed: aNumber
		"Create a new instance of the receiver with given seed
		  using D. Knuth's constants.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^( super new) initialize; setSeed: aNumber; yourself! !

!DhbCongruentialRandomNumberGenerator methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
floatValue
		"Answer the next pseudo-random value between 0 and 1.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^self value asFloat / modulus! !

!DhbCongruentialRandomNumberGenerator methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialize
		"Private - Initializes the constants of the receiver with D. Knuth's constants.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	self initialize: 2718281829.0 multiplicator: 3141592653.0 modulus: 4294967296.0.! !

!DhbCongruentialRandomNumberGenerator methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialize: aNumber1 multiplicator: aNumber2 modulus: aNumber3
		"Private - Initializes the constants needed by the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	constant := aNumber1.
	modulus := aNumber2.
	multiplicator := aNumber3.
	self setSeed: 1.! !

!DhbCongruentialRandomNumberGenerator methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
integerValue: anInteger
		"Answer a random integer between 0 and the anInteger.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	^( self value  \\ ( anInteger * 1000)) // 1000! !

!DhbCongruentialRandomNumberGenerator methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
setSeed: aNumber
		"Set the seed of the receiver to aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	seed := aNumber.! !

!DhbCongruentialRandomNumberGenerator methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
value
		"Answer the next pseudo-random value.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/2/99 "
	seed := ( seed * multiplicator + constant) \\ modulus.
	^seed! !

Object subclass: #DhbDecimalFloatingNumber
	instanceVariableNames: 'mantissa exponent'
	classVariableNames: 'Digits'
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbDecimalFloatingNumber commentStamp: '<historical>' prior: 0!
A DhbDecimalFloatingNumber is a demonstration of floating point arithmetic and rounding errors. The implementation uses decimal digits to allow easier human reading and understanding, and hand-worked examples. Only basic arithmetic operations are specified (+, -,*, sqrt). 

The message #value answers a fraction which is the exact representation of the stored number. 

The message #normalize uses truncation to fit the best precision possible. It is at this step that roundoff errors occur. #normalize is called when these numbers are created.

This class is detailed in appendix A of Didier Besset's book.!

!DhbDecimalFloatingNumber class methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
defaultDigits
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/6/99 "
	^15! !

!DhbDecimalFloatingNumber class methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
defaultDigits: anInteger
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/6/99 "
	Digits := anInteger.! !

!DhbDecimalFloatingNumber class methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
digits
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/6/99 "
	Digits isNil
		ifTrue: [ Digits := self defaultDigits].
	^Digits! !

!DhbDecimalFloatingNumber class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
new: aNumber
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/6/99 "
	^self new normalize: aNumber! !

!DhbDecimalFloatingNumber class methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
resetDigits
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/6/99 "
	Digits := nil.! !

!DhbDecimalFloatingNumber methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
* aNumber
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/6/99 "
	^self class new: ( self value * aNumber value)! !

!DhbDecimalFloatingNumber methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
+ aNumber
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/6/99 "
	^self class new: ( self value + aNumber value)! !

!DhbDecimalFloatingNumber methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
- aNumber
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/6/99 "
	^self class new: ( self value - aNumber value)! !

!DhbDecimalFloatingNumber methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
/ aNumber
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/6/99 "
	^self class new: ( self value / aNumber value)! !

!DhbDecimalFloatingNumber methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
normalize: aNumber
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/6/99 "
	exponent := (self class digits - (aNumber log: 10)) floor.
	mantissa := ( aNumber * ( 10 raisedToInteger: exponent)) truncated.
	^self! !

!DhbDecimalFloatingNumber methodsFor: 'display' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
printOn: aStream
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/6/99 "
	mantissa printOn: aStream.
	aStream nextPutAll: 'xE'.
	exponent negated printOn: aStream.! !

!DhbDecimalFloatingNumber methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
sqrt
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/6/99 "
	^self class new:  self value sqrt! !

!DhbDecimalFloatingNumber methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
value
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/6/99 "
	^mantissa / ( 10 raisedToInteger: exponent)! !

Object subclass: #DhbLUPDecomposition
	instanceVariableNames: 'rows permutation parity'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical-Math-Matrix'!

!DhbLUPDecomposition commentStamp: 'StephaneDucasse 7/5/2015 21:04' prior: 0!
LUP stands for Lower, Upper and Permutation. It comes from the observation that any non-singular square matrix A can be decomposed into a product of 3 square matrices of the same dimension.

LUP decomposition is another technique to solve a
system of linear equations. It is an alternative to the Gaussian elimination. Gaussian elimination can solve a system with several constant vectors, but all constant vectors must be known before starting the algorithm.

LUP decomposition is done once for the matrix of a given system. Thus, the system can be solved for any constant vector obtained after the LUP decomposition. In addition, LUP decomposition gives a way to calculate the determinant of a matrix and it can be used to compute the inverse of a matrix.

Instance variables
- rows contains a copy of the rows of the matrix representing the system of linear equations, i.e.the matrix A; copying the matrix is necessary since LUP decomposition destroys the components; at the end of the LUP decomposition, it will contain the components of the matrices L and U,
- permutation contains an array of integers describing the permutation, i.e.the matrix P,
- parity contains parity of the permutation for efficiency purpose.

[[[
| s sol1 sol2 |
s := DhbLUPDecomposition equations: #( (3 2 4) (2 -5 -1) (1 -2 2)).
sol1 := s solve: #(16 6 10).
sol2 := s solve: #(7 10 9).
]]]!

!DhbLUPDecomposition class methodsFor: 'creation' stamp: 'StephaneDucasse 7/5/2015 19:49'!
direct: anArrayOfArrays
	
	^ self basicNew initialize: anArrayOfArrays.! !

!DhbLUPDecomposition class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
equations: anArrayOfArrays
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 30/3/99 "
		^self new initialize: ( anArrayOfArrays collect: [ :each | each copy]).! !

!DhbLUPDecomposition methodsFor: 'private' stamp: 'StephaneDucasse 7/5/2015 19:47'!
backwardSubstitution: anArray
	"Private"
	| n sum answer |
	n := rows size.
	answer := DhbVector new: n.
	n to: 1 by: -1 do:
		[ :i |
		  sum := anArray at: i.
		  ( i + 1) to: n do: [ :j | sum := sum - ( ( ( rows at: i) at: j) * ( answer at: j))].
		  answer at: i put: sum / ( ( rows at: i) at: i).
		].
	^answer! !

!DhbLUPDecomposition methodsFor: 'private' stamp: 'StephaneDucasse 7/5/2015 19:52'!
decompose
	"Private"
	| n |
	n := rows size.
	permutation := (1 to: n) asArray.
	1 to: ( n - 1) do:
		[ :k |
		  self swapRow: k withRow: ( self largestPivotFrom: k);
			   pivotAt: k.
		].! !

!DhbLUPDecomposition methodsFor: 'operations' stamp: 'StephaneDucasse 7/5/2015 19:48'!
determinant
	| n |
	permutation isNil
		ifTrue: [ self protectedDecomposition ].
	permutation = 0
		ifTrue: [ ^ 0 ].	"Singular matrix has 0 determinant"
	n := 0.
	^ rows inject: parity
		  into: [ :det :each | n := n + 1. ( each at: n) * det ]! !

!DhbLUPDecomposition methodsFor: 'private' stamp: 'StephaneDucasse 7/5/2015 19:53'!
forwardSubstitution: anArray
	"Private"
	| n sum answer|
	answer := permutation collect: [ :each | anArray at: each].
	n := rows size.
	2 to: n do:
		[ :i |
		  sum := answer at: i.
		  1 to: (i - 1) do: [ :j | sum := sum - (((rows at: i) at: j) * (answer at: j))].
		  answer at: i put: sum.
		].
	^answer! !

!DhbLUPDecomposition methodsFor: 'initialization' stamp: 'StephaneDucasse 7/5/2015 19:49'!
initialize: anArrayOfArrays
	rows := anArrayOfArrays.
	parity := 1.
	^ self! !

!DhbLUPDecomposition methodsFor: 'operations' stamp: 'StephaneDucasse 7/5/2015 19:48'!
inverseMatrixComponents
	| n inverseRows column |
	permutation isNil
		ifTrue: [ self protectedDecomposition].
	permutation = 0
		ifTrue: [ ^ nil].	"Singular matrix has no inverse"
	n := rows size.
	inverseRows := (1 to: n) asDhbVector collect: [ :j | DhbVector new: n].
	1 to: n do:
		[ :j |
		  column := self solve: ( ( Array new: rows size) atAllPut: 0; at: j put: 1; yourself).
		  column ifNil: [^nil].	"Singular matrix has no inverse"
		  1 to: n do: [ :i | ( inverseRows at: i) at: j put: ( column at: i)].
		]. 
	^inverseRows! !

!DhbLUPDecomposition methodsFor: 'operations' stamp: 'StephaneDucasse 7/5/2015 19:49'!
largestPivotFrom: anInteger
	"Private - Answers the largest pivot element in column anInteger, from position anInteger upward."
	| valueOfMaximum indexOfMaximum value |
	valueOfMaximum := ((rows at: anInteger) at: anInteger) abs.
	indexOfMaximum := anInteger.
	(anInteger + 1) to: rows size do:
		[ :n |
		  value := ((rows at: n) at: anInteger) abs.
		  value > valueOfMaximum
				ifTrue: [ valueOfMaximum := value.
						  indexOfMaximum := n ].
		].
	^ indexOfMaximum! !

!DhbLUPDecomposition methodsFor: 'private' stamp: 'StephaneDucasse 7/5/2015 19:53'!
pivotAt: anInteger
	"Private"
	| inversePivot size k |
	inversePivot := 1 / ((rows at: anInteger) at: anInteger).
	size := rows size.
	k := anInteger + 1.
	k to: size
		do: [ :i |
			  (rows at: i) at: anInteger put: ((rows at: i) at: anInteger) * inversePivot.
			  k to: size
				do: [ :j |
					  (rows at: i) at: j put: ((rows at: i) at: j) - (((rows at: i) at: anInteger) * ((rows at: anInteger) at: j)).
					]
			].! !

!DhbLUPDecomposition methodsFor: 'printing' stamp: 'StephaneDucasse 7/5/2015 20:53'!
printOn: aStream
	
	| delimitingString |
	rows do:
		[ :row |
		  delimitingString := '('.
		  row do:
			[ :each |
				aStream nextPutAll: delimitingString.
				each printOn: aStream.
				delimitingString := ' ' ].
		  aStream nextPut: $).
		]
		separatedBy: [ aStream cr ]! !

!DhbLUPDecomposition methodsFor: 'private' stamp: 'StephaneDucasse 7/5/2015 19:52'!
protectedDecomposition
	"Private - If decomposition fails, set permutation to 0."
	[ self decompose] on: Error do: [ :signal | permutation := 0. signal return: nil].! !

!DhbLUPDecomposition methodsFor: 'private' stamp: 'StephaneDucasse 7/5/2015 19:53'!
solve: anArrayOrVector
	permutation isNil
		ifTrue: [ self protectedDecomposition ].
	^ permutation = 0
		ifTrue: [ nil ]
		ifFalse:[ 
			[self backwardSubstitution: ( self forwardSubstitution: anArrayOrVector)] on: ZeroDivide do: [:e|  nil] ]! !

!DhbLUPDecomposition methodsFor: 'private' stamp: 'StephaneDucasse 7/5/2015 19:51'!
swapRow: anInteger1 withRow: anInteger2
	"Private - Swap the rows indexed by the given integers."
	| swappedRow |
	anInteger1 = anInteger2 ifTrue: [ ^ self ].
	swappedRow := rows at: anInteger1.
	rows at: anInteger1 put: (rows at: anInteger2).
	rows at: anInteger2 put: swappedRow.
	swappedRow := permutation at: anInteger1.
	permutation at: anInteger1 put: (permutation at: anInteger2).
	permutation at: anInteger2 put: swappedRow.
	parity := parity negated! !

Object subclass: #DhbLagrangeInterpolator
	instanceVariableNames: 'pointCollection'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbLagrangeInterpolator commentStamp: '<historical>' prior: 0!
A DhbLagrangeInterpolator takes a set of points (include each with #add:) and interpolates values using the Lagrange Interpolation Polynomial. This is the polynomial of minimum degree through all points in the pointCollection.

This is appropriate for interpolation only (interior to the region of the set of values), and sensitive to several conditions.

The method value: yields an approximation at a given value of the independent variable. Calculation is deferred until value is sent. If the set of points is fixed, NewtonInterpolator precomputes the value function. If the set of points is likely to change more often than interpolated values are needed, this is a fair choice.!

!DhbLagrangeInterpolator class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
new
		"Create a new instance of the receiver without points. Points must be added with add:
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^super new initialize! !

!DhbLagrangeInterpolator class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
points: aCollectionOfPoints
		"Create a new instance of the receiver with given points.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^self new initialize: aCollectionOfPoints! !

!DhbLagrangeInterpolator methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
add: aPoint
		"Add a point to the collection of points.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^pointCollection add: aPoint! !

!DhbLagrangeInterpolator methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
defaultSamplePoints
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 27/5/99 "
	^OrderedCollection new! !

!DhbLagrangeInterpolator methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialize
		"Private - Create an empty point collection for the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^self initialize: self defaultSamplePoints! !

!DhbLagrangeInterpolator methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialize: aCollectionOfPoints
		"Private - Defines the collection of points for the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	pointCollection := aCollectionOfPoints.
	^self! !

!DhbLagrangeInterpolator methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
size
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 3/12/00 "
	^pointCollection size! !

!DhbLagrangeInterpolator methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
value: aNumber
		"Compute the value of the Lagrange interpolation polynomial on the receiver's points at aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	| norm dx products answer size |
	norm := 1.
	size := pointCollection size.
	products := Array new: size.
	products atAllPut: 1.
	1 to: size
		do: [ :n |
			  dx := aNumber - ( self xPointAt: n).
			  dx = 0
				ifTrue: [ ^( self yPointAt: n)].
			  norm := norm * dx.
			  1 to: size
				do: [ :m |
					  m = n
						ifFalse:[ products at: m put: ( (( self xPointAt: m) - ( self xPointAt: n)) * ( products at: m))].
					].
			].
	answer := 0.
	1 to: size do:
		[ :n | answer := ( self yPointAt: n) / ( ( products at: n) * ( aNumber - ( self xPointAt: n))) + answer].
	^norm * answer! !

!DhbLagrangeInterpolator methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
xPointAt: anInteger
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 19/4/99 "
	^( pointCollection at: anInteger) x! !

!DhbLagrangeInterpolator methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
yPointAt: anInteger
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 19/4/99 "
	^( pointCollection at: anInteger) y! !

DhbLagrangeInterpolator subclass: #DhbNevilleInterpolator
	instanceVariableNames: 'leftErrors rightErrors'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

DhbNevilleInterpolator subclass: #DhbBulirschStoerInterpolator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbBulirschStoerInterpolator methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
computeDifference: aNumber at: anInteger1 order: anInteger2
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 28/5/99 "
	| diff ratio |
	ratio := ( ( self xPointAt: anInteger1) - aNumber) * ( rightErrors at: anInteger1)
							/ ( (  self xPointAt: ( anInteger1 + anInteger2)) - aNumber).
	diff := ( ( leftErrors at: ( anInteger1 + 1)) - ( rightErrors at: anInteger1))
							/ ( ratio - ( leftErrors at: ( anInteger1 + 1))).
	rightErrors at: anInteger1 put: ( leftErrors at: ( anInteger1 + 1)) * diff.	
	leftErrors at: anInteger1 put: ratio * diff.! !

!DhbNevilleInterpolator methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
computeDifference: aNumber at: anInteger1 order: anInteger2
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 28/5/99 "
	| leftDist rightDist ratio |
	leftDist := ( self xPointAt: anInteger1) - aNumber.
	rightDist := (  self xPointAt: ( anInteger1 + anInteger2)) - aNumber.
	ratio := ( ( leftErrors at: ( anInteger1 + 1)) - ( rightErrors at: anInteger1)) / ( leftDist - rightDist).
	leftErrors at: anInteger1 put: ratio * leftDist.
	rightErrors at: anInteger1 put: ratio * rightDist.! !

!DhbNevilleInterpolator methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
defaultSamplePoints
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 27/5/99 "
	^SortedCollection sortBlock: [ :a :b | a x < b x]! !

!DhbNevilleInterpolator methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialize
	
	super initialize.
	leftErrors := Array new.
	rightErrors := Array new! !

!DhbNevilleInterpolator methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initializeDifferences: aNumber
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 28/5/99 "
	| size nearestIndex dist minDist |
	size := pointCollection size.
	leftErrors size = size
		ifFalse:[ leftErrors := Array new: size.
				  rightErrors := Array new: size.
				].
	minDist := ( ( self xPointAt: 1) - aNumber) abs.
	nearestIndex := 1.
	leftErrors at: 1 put: ( self yPointAt: 1).
	rightErrors at: 1 put: leftErrors first.
	2 to: size do:
		[ :n |
		  dist := ( ( self xPointAt: n) - aNumber) abs.
		  dist < minDist
			ifTrue: [ dist = 0
						ifTrue: [ ^n negated].
					  nearestIndex := n.
					  minDist := dist.
					].
		 leftErrors at: n put: ( self yPointAt: n).
		 rightErrors at: n put: ( leftErrors at: n).
		].
	^nearestIndex! !

!DhbNevilleInterpolator methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
value: aNumber
		"Compute the value of the Lagrange interpolation polynomial on the receiver's points at aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 7/4/99 "
	^(self valueAndError: aNumber) first! !

!DhbNevilleInterpolator methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
valueAndError: aNumber
		"Compute and return the interpolated value of the interpolation Lagranage polynomial
		 and its estimated error.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 7/4/99 "
	| size nearestIndex answer error |
	nearestIndex := self initializeDifferences: aNumber.
	nearestIndex < 0
		ifTrue: [ ^Array with: ( self yPointAt: nearestIndex negated) with: 0].
	answer := leftErrors at: nearestIndex.
	nearestIndex := nearestIndex - 1.
	size := pointCollection size.
	1 to: ( size - 1) do:
		[ :m |
		  1 to: ( size - m) do:
			[ :n | self computeDifference: aNumber at: n order: m].
		  size - m > ( 2 * nearestIndex)
				ifTrue: [ error := leftErrors at: ( nearestIndex + 1) ]
				ifFalse:[ error := rightErrors at: ( nearestIndex).
							  nearestIndex := nearestIndex - 1.
							].
		  answer := answer + error.
		].
	^Array with: answer with: error abs! !

DhbLagrangeInterpolator subclass: #DhbNewtonInterpolator
	instanceVariableNames: 'coefficients'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbNewtonInterpolator commentStamp: '<historical>' prior: 0!
A DhbNewtonInterpolator is a specialized Lagrange Interpolator which precomputes the polynomial to be evaluated when interpolating. This is twice as expensive as direct evaluation, but produces a linear time (in the number of points) evaluation method for fixed points.

add: resets the coefficients

value: lazily initializes the coefficients and yields an interpolated value.

computeCoefficients and resetCoefficients should be considered private.!

!DhbNewtonInterpolator methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
add: aPoint
		"Add a point to the collection of points.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	self resetCoefficients.
	^super add: aPoint! !

!DhbNewtonInterpolator methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
computeCoefficients
		"Private - Computes the coefficients for the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	| size k1 kn|
	size := pointCollection size.
	coefficients := ( 1 to: size) collect: [ :n | self yPointAt: n].
	1 to: (size - 1)
		do: [ :n |
			  size to: ( n + 1)  by: -1
				do: [ :k |
					  k1 := k - 1.
					  kn := k - n.
					  coefficients at: k put: ( (( coefficients at: k) - ( coefficients at: k1)) 
											/ ((self xPointAt: k) - (self xPointAt: kn))).
					].
			].! !

!DhbNewtonInterpolator methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
resetCoefficients
		"Private - Reset the coefficients of the receiver to force a new computation.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	coefficients := nil.! !

!DhbNewtonInterpolator methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
value: aNumber
		"Compute the value of the Lagrange interpolation polynomial on the receiver's points at aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	| answer size |
	coefficients isNil
		ifTrue: [ self computeCoefficients].
	size := coefficients size.
	answer := coefficients at: size.
	(size - 1) to: 1 by: -1
		do: [ :n | answer := answer * ( aNumber - (self xPointAt:  n)) + ( coefficients at: n)].
	^answer! !

DhbNewtonInterpolator subclass: #DhbSplineInterpolator
	instanceVariableNames: 'startPointDerivative endPointDerivative'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbSplineInterpolator methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
computeSecondDerivatives
	"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 7/4/99 "

	| size u w s dx inv2dx |
	size := pointCollection size.
	coefficients := Array new: size.
	u := Array new: size - 1.
	startPointDerivative isNil 
		ifTrue: 
			[coefficients at: 1 put: 0.
			u at: 1 put: 0]
		ifFalse: 
			[coefficients at: 1 put: -1 / 2.
			s := 1 / (( self xPointAt: 2) x - ( self xPointAt: 1) x).
			u at: 1
				put: 3 * s 
						* (s * (( self yPointAt: size) - ( self yPointAt: size - 1)) 
								- startPointDerivative)].
	2 to: size - 1
		do: 
			[:n | 
			dx := (self xPointAt: n) - (self xPointAt: ( n - 1)).
			inv2dx := 1 / (( self xPointAt: n + 1) - (self xPointAt: n - 1)).
			s := dx * inv2dx.
			w := 1 / (s * (coefficients at: n - 1) + 2).
			coefficients at: n put: (s - 1) * w.
			u at: n
				put: (((( self yPointAt: n + 1) - ( self yPointAt: n)) 
						/ (( self xPointAt: n + 1) - ( self xPointAt: n)) 
							- ((( self yPointAt: n) - ( self yPointAt: n - 1)) / dx)) * 6 
						* inv2dx - ((u at: n - 1) * s)) 
						* w].
	endPointDerivative isNil 
		ifTrue: [coefficients at: size put: 0]
		ifFalse: 
			[w := 1 / 2.
			s := 1 / ((self xPointAt:  size) - (self xPointAt: ( size - 1))).
			u at: 1
				put: 3 * s * (endPointDerivative 
								- (s * (self yPointAt: size) - (self yPointAt: size - 1))).
			coefficients at: size
				put: s - (w * (u at: size - 1) / ((coefficients at: size - 1) * w + 1))].
	size - 1 to: 1
		by: -1
		do: 
			[:n | 
			coefficients at: n
				put: (coefficients at: n) * (coefficients at: n + 1) + (u at: n)]! !

!DhbSplineInterpolator methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
defaultSamplePoints
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 27/5/99 "
	^SortedCollection sortBlock: [ :a :b | a x < b x]! !

!DhbSplineInterpolator methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
endPointDerivative: aNumber
		"Defines the end point derivatives.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 7/4/99 "
	endPointDerivative := aNumber.
	self resetCoefficients.! !

!DhbSplineInterpolator methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
resetEndPointDerivatives
		"Set the end point derivatives to undefined.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 7/4/99 "
	self setEndPointDerivatives: ( Array new: 2).! !

!DhbSplineInterpolator methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
setEndPointDerivatives: anArray
		"Defines the end point derivatives.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 7/4/99 "
	startPointDerivative := anArray at: 1.
	endPointDerivative := anArray at: 2.
	self resetCoefficients.! !

!DhbSplineInterpolator methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
startPointDerivative: aNumber
		"Defines the end point derivatives.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 7/4/99 "
	startPointDerivative := aNumber.
	self resetCoefficients.! !

!DhbSplineInterpolator methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
value: aNumber 
	"Computes the value of a cubic spline interpolation over the points of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 7/4/99 "

	| n1 n2 n step a b |
	coefficients isNil ifTrue: [self computeSecondDerivatives].
	n2 := pointCollection size.
	n1 := 1.
	[n2 - n1 > 1] whileTrue: 
			[n := (n1 + n2) // 2.
			(self xPointAt:  n) > aNumber ifTrue: [n2 := n] ifFalse: [n1 := n]].
	step := (self xPointAt: n2) - (self xPointAt: n1).
	a := ((self xPointAt: n2) - aNumber) / step.
	b := (aNumber - (self xPointAt: n1)) / step.
	^a * (self yPointAt: n1) + (b * (self yPointAt: n2)) 
		+ ((a * (a squared - 1) * (coefficients at: n1) 
				+ (b * (b squared - 1) * (coefficients at: n2))) * step squared 
				/ 6)! !

Object subclass: #DhbLanczosFormula
	instanceVariableNames: 'coefficients sqrt2Pi'
	classVariableNames: 'UniqueInstance'
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbLanczosFormula commentStamp: '<historical>' prior: 0!
A DhbLanczosFormula is a singleton class which calculates an approximation to Gamma(x). 
Gamma function is a continuous extension to Factorial. Gamma(x + 1) = x * Gamma(x). For integers Gamma(n) = (n-1) factorial. 

This is called from Number>>gamma, and Number>>logGamma.

Instance variable coefficients contains the terms of the approximation.


The method is detailed in Numerical Recipes in C. The implementation is detailed in Besset's book Section 2.4
!

!DhbLanczosFormula class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
new
		"Answer a unique instance. Create it if it does not exist.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/1/99 "
	UniqueInstance isNil
		ifTrue: [ UniqueInstance := super new.
					 UniqueInstance initialize.
					].
	^UniqueInstance! !

!DhbLanczosFormula class methodsFor: 'instance creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
reset
	UniqueInstance := nil.! !

!DhbLanczosFormula methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
gamma: aNumber
		"
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 1/3/99 "
	^( self leadingFactor: aNumber) exp * ( self series: aNumber) * sqrt2Pi / aNumber! !

!DhbLanczosFormula methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialize
		"Private - Initialize the coefficients of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 1/3/99 "
	sqrt2Pi := ( Float pi * 2) sqrt.
	coefficients := #( 76.18009172947146 -86.50532032941677 24.01409824083091 -1.231739572450155 0.1208650973866179e-2 -0.5395239384953e-5).
	^self! !

!DhbLanczosFormula methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
leadingFactor: aNumber
		"Private - Answers the log of the leading factor in Lanczos' formula.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 1/3/99 "
	| temp |
	temp := aNumber + 5.5.
	^( temp ln * ( aNumber + 0.5) - temp)! !

!DhbLanczosFormula methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
logGamma: aNumber
		"
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 1/3/99 "
	^( self leadingFactor: aNumber) + ( ( self series: aNumber) * sqrt2Pi / aNumber) ln! !

!DhbLanczosFormula methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
series: aNumber
		"Private - Answer the value of the series of Lanczos' formula.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 1/3/99 "
	| term |
	term := aNumber.
	^coefficients inject: 1.000000000190015
						into: [ :sum :each | term := term + 1. each / term + sum]! !

Object subclass: #DhbLinearEquationSystem
	instanceVariableNames: 'rows solutions'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical-Math-Matrix'!

!DhbLinearEquationSystem commentStamp: 'StephaneDucasse 7/5/2015 21:35' prior: 0!
This class offers Gaussian elimination.

[[[ 
 (DhbLinearEquationSystem equations: #( (3 2 4)
                                   (2 -5 -1)
                                   (1 -2 2))
                      constant: #(16 6 10)
     ) solution.
]]]

Note that Gaussian elimination is solely dedicated to solving systems of linear equations. The algorithm is somewhat slower than LUP decomposition.

If the system does not have a solution --- that is, if the system's matrix is singular --- an arithmetic error occurs in the
method ==pivotAt:== when the division with the zero pivot is performed. The method ==solutionAt:== traps this error within an
exception handling structure and sets the solution vector to a special value --- the integer 0 --- as a flag to prevent attempting Gaussian elimination a second time. Then, the value ==nil== is returned to represent the non-existent solution.!

!DhbLinearEquationSystem class methodsFor: 'creation' stamp: 'StephaneDucasse 7/5/2015 20:03'!
equations: anArrayOfArrays constant: anArray 

	^ self basicNew 
			initialize: anArrayOfArrays 
			constants: (Array with: anArray)! !

!DhbLinearEquationSystem class methodsFor: 'creation' stamp: 'StephaneDucasse 7/5/2015 20:03'!
equations: anArrayOfArrays constants: anArrayOfConstantArrays 

	^ self basicNew 
		initialize: anArrayOfArrays 
		constants: anArrayOfConstantArrays! !

!DhbLinearEquationSystem methodsFor: 'transformation' stamp: 'StephaneDucasse 7/5/2015 20:12'!
backSubstitutionAt: anInteger
		"Private - Perform the back-substitution step corresponding to the anInteger-th constant array."
	| size answer accumulator |
	size := rows size.
	answer := Array new: size.
	size to: 1 by: -1 do:
		[ :n |
		  accumulator := (rows at: n) at: (anInteger + size).
		  (n + 1) to: size
			do: [ :m | accumulator := accumulator - ((answer at: m) * ((rows at: n) at: m))].
		  answer at: n put: (accumulator / ( ( rows at: n) at: n)).
		].
	solutions at: anInteger put: answer.! !

!DhbLinearEquationSystem methodsFor: 'initialization' stamp: 'StephaneDucasse 7/5/2015 20:04'!
initialize: anArrayOfArrays constants: anArrayOfConstantArrays
	"Private - Initialize the receiver with system's matrix in anArrayOfArrays and several constants."
	| n |
	n := 0.
	rows := anArrayOfArrays collect: [ :each | n := n + 1. each, (anArrayOfConstantArrays collect: [ :c | c at: n])].
	^ self! !

!DhbLinearEquationSystem methodsFor: 'information' stamp: 'StephaneDucasse 7/5/2015 20:11'!
largestPivotFrom: anInteger
	"Private - Answers the largest pivot element in column anInteger, from position anInteger upward."
	| valueOfMaximum indexOfMaximum |
	valueOfMaximum := (rows at: anInteger) at: anInteger.
	indexOfMaximum := anInteger.
	(anInteger + 2) to: rows size do:
		[ :n |
		  ((rows at: n) at: anInteger) > valueOfMaximum
				ifTrue: [ valueOfMaximum := (rows at: n) at: anInteger.
							 indexOfMaximum := n.
						   ].
		].
	^ indexOfMaximum! !

!DhbLinearEquationSystem methodsFor: 'transformation' stamp: 'StephaneDucasse 7/5/2015 20:07'!
pivotAt: anInteger
	"Private - Performs pivot operation with pivot element at anInteger."
	| inversePivot rowPivotValue row pivotRow |
	pivotRow := rows at: anInteger.
	inversePivot := 1 / (pivotRow at: anInteger).
	(anInteger + 1) to: rows size do:
		[ :n |
		  row := rows at: n.
		  rowPivotValue := (row at: anInteger) * inversePivot.
		  anInteger to: row size do:
			[ :m |
			  row at: m put: ((row at: m) - ((pivotRow at: m) * rowPivotValue)).
			].
		].! !

!DhbLinearEquationSystem methodsFor: 'transformation' stamp: 'StephaneDucasse 7/5/2015 20:07'!
pivotStepAt: anInteger
	"Private - Performs an optimum pivot operation at anInteger."
	self swapRow: anInteger withRow: (self largestPivotFrom: anInteger);
		   pivotAt: anInteger.! !

!DhbLinearEquationSystem methodsFor: 'printing' stamp: 'StephaneDucasse 7/5/2015 20:11'!
printOn: aStream
	"Append to the argument aStream, a sequence of characters that describes the receiver."
	| delimitingString n k |
	n := rows size.
	rows 
		do:
			[ :row |
			  delimitingString := '('.
			  k := 0.
			  row do:
				[ :each |
					aStream nextPutAll: delimitingString.
					each printOn: aStream.
					k := k + 1.
					delimitingString := k < n ifTrue: [ ' '] ifFalse: [ ' : '].
				].
		 	aStream nextPut: $).]
		separatedBy: [ aStream cr]
	! !

!DhbLinearEquationSystem methodsFor: 'information' stamp: 'StephaneDucasse 7/5/2015 20:02'!
solution
	"Answers the solution corresponding to the first constant array."
	^ self solutionAt: 1! !

!DhbLinearEquationSystem methodsFor: 'information' stamp: 'StephaneDucasse 7/5/2015 20:01'!
solutionAt: anInteger
	"Answer the solution corresponding to the anInteger-th constant array."

	solutions isNil
		ifTrue: [ [self solve] on: Error do: [ :signal |solutions := 0. signal return: nil.] ].
	solutions = 0
		ifTrue: [ ^nil].
	(solutions at: anInteger) isNil
		ifTrue: [ self backSubstitutionAt: anInteger].
	^ solutions at: anInteger! !

!DhbLinearEquationSystem methodsFor: 'transformation' stamp: 'StephaneDucasse 7/5/2015 20:01'!
solve
	"Private - Perform LU decomposition of the system."
	1 to: rows size do: [ :n | self pivotStepAt: n ].
	solutions := Array new: ((rows at: 1) size - rows size).! !

!DhbLinearEquationSystem methodsFor: 'transformation' stamp: 'StephaneDucasse 7/5/2015 20:01'!
swapRow: anInteger1 withRow: anInteger2
	"Private - Swap the rows indexed by the given integers."

	| swappedRow |
	anInteger1 = anInteger2
		ifTrue: [ ^ self ].
	swappedRow := rows at: anInteger1.
	rows at: anInteger1 put: (rows at: anInteger2).
	rows at: anInteger2 put: swappedRow! !

Object subclass: #DhbLinearRegression
	instanceVariableNames: 'sum1 sumX sumY sumXX sumYY sumXY slope intercept correlationCoefficient'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbLinearRegression class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
new
		"Create a new instance of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/2/99 "
	^( super new) reset; yourself! !

!DhbLinearRegression methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
add: aPoint
		"Accumulate aPoint into of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/2/99 "
	self add: aPoint weight: 1.! !

!DhbLinearRegression methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
add: aPoint weight: aNumber 
	"Accumulate aPoint into of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/2/99 "

	sum1 := sum1 + aNumber.
	sumX := sumX + (aPoint x * aNumber).
	sumY := sumY + (aPoint y * aNumber).
	sumXX := sumXX + (aPoint x squared * aNumber).
	sumYY := sumYY + (aPoint y squared * aNumber).
	sumXY := sumXY + (aPoint x * aPoint y * aNumber).
	self resetResults! !

!DhbLinearRegression methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
asEstimatedPolynomial
		"Answer the resulting linear dependence found by the receiver in the form of a polynomial
		 with embedded error matrix.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 18/3/99 "
	^( DhbEstimatedPolynomial coefficients: self coefficients)
			errorMatrix: self errorMatrix;
			yourself! !

!DhbLinearRegression methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
asPolynomial
		"Answer the resulting linear dependence found by the receiver in the form of a polynomial.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/2/99 "
	^DhbPolynomial coefficients: self coefficients! !

!DhbLinearRegression methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
coefficients
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 18/3/99 "
	^Array with: self intercept with: self slope! !

!DhbLinearRegression methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
computeResults
	"Private - Compute the results of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/2/99 "

	| xNorm xyNorm |
	xNorm := sumXX * sum1 - (sumX * sumX).
	xyNorm := sumXY * sum1 - (sumX * sumY).
	slope := xyNorm / xNorm.
	intercept := (sumXX * sumY - (sumXY * sumX)) / xNorm.
	correlationCoefficient := xyNorm 
				/ (xNorm * (sumYY * sum1 - (sumY * sumY))) sqrt! !

!DhbLinearRegression methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
correlationCoefficient
		"Answers the correlation coefficient of the receiver
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/2/99 "
	correlationCoefficient isNil
		ifTrue: [ self computeResults].
	^correlationCoefficient! !

!DhbLinearRegression methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
errorMatrix
	"Answer the resulting linear dependence found by the receiver in the form of a polynomial
		 with embedded error matrix.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 18/3/99 "

	| c1 cx cxx |
	c1 := 1.0 / (sumXX * sum1 - sumX squared).
	cx := sumX negated * c1.
	cxx := sumXX * c1.
	c1 := sum1 * c1.
	^DhbSymmetricMatrix rows: (Array with: (Array with: cxx with: cx)
				with: (Array with: cx with: c1))! !

!DhbLinearRegression methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
errorOnIntercept
	"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 21/5/99 "

	^(sumXX / (sumXX * sum1 - sumX squared)) sqrt! !

!DhbLinearRegression methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
errorOnSlope
	"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 21/5/99 "

	^(sum1 / (sumXX * sum1 - sumX squared)) sqrt! !

!DhbLinearRegression methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
intercept
		"Answers the intercept of the receiver
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/2/99 "
	intercept isNil
		ifTrue: [ self computeResults].
	^intercept! !

!DhbLinearRegression methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
remove: aPoint 
	"Remove aPoint which was accumulated into of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/2/99 "

	sum1 := sum1 - 1.
	sumX := sumX - aPoint x.
	sumY := sumY - aPoint y.
	sumXX := sumXX - aPoint x squared.
	sumYY := sumYY - aPoint y squared.
	sumXY := sumXY - (aPoint x * aPoint y).
	self resetResults! !

!DhbLinearRegression methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
reset
	"Set all accumulators of the receiver to zero and reset its results.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/2/99 "

	sum1 := 0.
	sumX := 0.
	sumY := 0.
	sumXX := 0.
	sumYY := 0.
	sumXY := 0.
	self resetResults! !

!DhbLinearRegression methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
resetResults
		"Private - Reset the results of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/2/99 "
	slope := nil.
	intercept := nil.
	correlationCoefficient := nil.! !

!DhbLinearRegression methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
slope
		"Answers the slope of the receiver
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/2/99 "
	slope isNil
		ifTrue: [ self computeResults].
	^slope! !

!DhbLinearRegression methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
value: aNumber
		"Answer the value interpolated at aNumber by the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/2/99 "
	^aNumber * self slope + self intercept! !

Object subclass: #DhbMahalanobisCenter
	instanceVariableNames: 'center inverseCovariance accumulator'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbMahalanobisCenter class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
new: anInteger
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	^self new initialize: anInteger! !

!DhbMahalanobisCenter class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
onVector: aVector
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	^self new center: aVector! !

!DhbMahalanobisCenter methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
accumulate: aVector
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	accumulator accumulate: aVector.! !

!DhbMahalanobisCenter methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
center: aVector
		"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	accumulator := DhbCovarianceAccumulator new: aVector size.
	center := aVector.
	inverseCovariance := DhbSymmetricMatrix identity: aVector size.
	^self! !

!DhbMahalanobisCenter methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
computeParameters
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	center := accumulator average copy.
	inverseCovariance := accumulator covarianceMatrix inverse.! !

!DhbMahalanobisCenter methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
count
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	^accumulator count! !

!DhbMahalanobisCenter methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
distanceTo: aVector
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	| delta |
	delta := aVector - center.
	^delta * inverseCovariance * delta! !

!DhbMahalanobisCenter methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialize: anInteger
		"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	accumulator := DhbCovarianceAccumulator new: anInteger.
	^self! !

!DhbMahalanobisCenter methodsFor: 'display' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
printOn: aStream
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	accumulator count printOn: aStream.
	aStream nextPutAll: ': '.
	center printOn: aStream.! !

!DhbMahalanobisCenter methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
reset
		"Leave center and inverse covariant matrix untouched
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	accumulator reset.! !

Object subclass: #DhbMatrix
	instanceVariableNames: 'rows lupDecomposition'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical-Math-Matrix'!

!DhbMatrix commentStamp: 'StephaneDucasse 7/5/2015 19:14' prior: 0!
I represent a mathematical matrix. I can be build from rows as follows: 

[[[ 
DhbMatrix rows: #((1 2 3)(4 5 6)).
]]]

I understand the usual matrix operations.
!

!DhbMatrix class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
join: anArrayOfMatrices
		"Inverse of the split operation.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 8/5/99 "
	| rows n row rowSize n1 n2 |
	rows := OrderedCollection new.
	n1 := ( anArrayOfMatrices at: 1) numberOfColumns.
	n2 := n1 + 1.
	rowSize := n1 + ( anArrayOfMatrices at: 2) numberOfColumns.
	n := 0.
	( anArrayOfMatrices at: 1) rowsDo:
		[ :each |
		  n := n + 1.
		  row := DhbVector new: rowSize.
		  row replaceFrom: 1 to: n1 with: each startingAt: 1;
			  replaceFrom: n2 to: rowSize with: ( ( anArrayOfMatrices at: 2) rowAt: n) startingAt: 1.
		  rows add: row.
		].
	n := 0.
	( anArrayOfMatrices at: 3) rowsDo:
		[ :each |
		  n := n + 1.
		  row := DhbVector new: rowSize.
		  row replaceFrom: 1 to: n1 with: each startingAt: 1;
			  replaceFrom: n2 to: rowSize with: ( ( anArrayOfMatrices at: 4) rowAt: n) startingAt: 1.
		  rows add: row.
		].
	^self rows: rows! !

!DhbMatrix class methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
lupCRLCriticalDimension
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/5/99 "
	^40! !

!DhbMatrix class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
new: anInteger
		"Create an empty square matrix of dimension anInteger.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^self new initialize: anInteger! !

!DhbMatrix class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
rows: anArrayOrVector
		"Create a new matrix with given components.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/2/99 "
	^self new initializeRows: anArrayOrVector! !

!DhbMatrix methodsFor: 'operation' stamp: 'StephaneDucasse 7/5/2015 19:13'!
* aNumberOrMatrixOrVector
	"Answers the product of the receiver with the argument. The argument can be a number, matrix or vector."
	^ aNumberOrMatrixOrVector productWithMatrix: self! !

!DhbMatrix methodsFor: 'operation' stamp: 'StephaneDucasse 7/5/2015 19:13'!
+ aMatrix
	"Answers the sum of the receiver with aMatrix."
	^ aMatrix addWithRegularMatrix: self! !

!DhbMatrix methodsFor: 'operation' stamp: 'StephaneDucasse 7/5/2015 19:14'!
- aMatrix
	"Answers the difference between the receiver and aMatrix."
	^ aMatrix subtractWithRegularMatrix: self! !

!DhbMatrix methodsFor: 'comparing' stamp: 'StephaneDucasse 7/5/2015 19:04'!
= aNumberOrMatrix 
	^ (aNumberOrMatrix species = self species) and: [ self rows = aNumberOrMatrix rows ]! !

!DhbMatrix methodsFor: 'double dispatching' stamp: 'StephaneDucasse 7/5/2015 19:04'!
adaptToNumber: rcvr andSend: selector
	"selector must obviously be commutative for this simple solution, but at the moment its only used for multiplication"
	^ self perform:  selector with: rcvr.! !

!DhbMatrix methodsFor: 'double dispatching' stamp: 'StephaneDucasse 7/5/2015 19:05'!
addWithRegularMatrix: aMatrix
	"Answers the sum of the receiver with aMatrix as a DhbMatrix."
	| n |
	n := 0.
	^ DhbMatrix rows: ( self rowsCollect: [ :each | n := n + 1. each + ( aMatrix rowAt: n)])! !

!DhbMatrix methodsFor: 'double dispatching' stamp: 'StephaneDucasse 7/5/2015 19:05'!
addWithSymmetricMatrix: aMatrix
	^ aMatrix addWithRegularMatrix: self ! !

!DhbMatrix methodsFor: 'transformation' stamp: 'StephaneDucasse 7/5/2015 19:27'!
asSymmetricMatrix
	"Convert the receiver to a symmetric matrix (no check is made)."
	^ DhbSymmetricMatrix rows: rows! !

!DhbMatrix methodsFor: 'accessing' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
atColumn: aColumnIndex put: aCollection

	aCollection withIndexDo: [: value : rowIndex |
		self rowAt: rowIndex columnAt: aColumnIndex put: value ]
	
! !

!DhbMatrix methodsFor: 'accessing' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
atRow: rowIndex put: aCollection

	aCollection withIndexDo: [: value : columnIndex |
		self rowAt: rowIndex columnAt: columnIndex put: value ]
	
! !

!DhbMatrix methodsFor: 'accessing' stamp: 'StephaneDucasse 7/5/2015 19:08'!
columnAt: anInteger
	"Answers the anInteger-th column of the receiver."
	^ rows collect: [ :each | each at: anInteger ]! !

!DhbMatrix methodsFor: 'iterators' stamp: 'StephaneDucasse 7/5/2015 19:12'!
columnsCollect: aBlock
	"Perform the collect: operation on the rows of the receiver."
	| n |
	n := 0.
	^ rows last collect: [ :each | n := n + 1. aBlock value: (self columnAt: n)]! !

!DhbMatrix methodsFor: 'iterators' stamp: 'StephaneDucasse 7/5/2015 19:12'!
columnsDo: aBlock
	"Perform the collect: operation on the rows of the receiver."
	| n |
	n := 0.
	^ rows last do: [ :each | n := n + 1. aBlock value: ( self columnAt: n)]! !

!DhbMatrix methodsFor: 'accessing' stamp: 'StephaneDucasse 7/5/2015 19:08'!
determinant
	^ self lupDecomposition determinant! !

!DhbMatrix methodsFor: 'comparing' stamp: 'StephaneDucasse 7/5/2015 19:04'!
hash
	^ rows hash! !

!DhbMatrix methodsFor: 'initialization' stamp: 'StephaneDucasse 7/5/2015 19:11'!
initialize: anInteger
	"Build empty components for a square matrix. No check is made: components are assumed to be orgainized in rows."
	rows := (1 to: anInteger) asDhbVector collect: [ :each | DhbVector new: anInteger].! !

!DhbMatrix methodsFor: 'initialization' stamp: 'StephaneDucasse 7/5/2015 19:12'!
initializeRows: anArrayOrVector
	"Defines the components of the recevier. No check is made: components are assumed to be orgainized in rows."
	rows := anArrayOrVector asDhbVector collect: [ :each | each asDhbVector].! !

!DhbMatrix methodsFor: 'operation' stamp: 'StephaneDucasse 7/5/2015 19:15'!
inverse
	"Answer the inverse of the receiver."

	^ self isSquare 
		ifTrue: [ self lupInverse ]
		ifFalse: [ self squared inverse * self transpose ]! !

!DhbMatrix methodsFor: 'operation' stamp: 'StephaneDucasse 7/5/2015 19:15'!
inversePureCRL
	"Answer the inverse of the receiver."
	^ self squared inversePureCRL * self transpose! !

!DhbMatrix methodsFor: 'testing' stamp: 'StephaneDucasse 7/5/2015 19:08'!
isSquare
	"Answers true if the number of rows is equal to the number of columns."
	^ rows size = rows last size! !

!DhbMatrix methodsFor: 'testing' stamp: 'StephaneDucasse 7/5/2015 19:08'!
isSymmetric
	^ self = self transpose ! !

!DhbMatrix methodsFor: 'private' stamp: 'StephaneDucasse 7/5/2015 19:08'!
largestPowerOf2SmallerThan: anInteger
	"Private - "
	| m m2|
	m := 2.
	[ m2 := m * 2.
	  m2 < anInteger] whileTrue:[ m := m2].
	^m! !

!DhbMatrix methodsFor: 'accessing' stamp: 'StephaneDucasse 7/5/2015 19:08'!
lupDecomposition

	lupDecomposition isNil
		ifTrue: [ lupDecomposition :=DhbLUPDecomposition equations: rows ].
	^ lupDecomposition! !

!DhbMatrix methodsFor: 'operation' stamp: 'WernerKassens 5/2/2014 08:46'!
lupInverse
	self lupDecomposition inverseMatrixComponents 
		ifNil: [ SingularMatrixError new signal ] 
		ifNotNilDo: [ :i| ^self class rows:i ].
! !

!DhbMatrix methodsFor: 'transformation' stamp: 'StephaneDucasse 7/5/2015 19:27'!
negate
	"Inverse the sign of all components of the receiver."
	rows do: [ :each |each negate ]! !

!DhbMatrix methodsFor: 'accessing' stamp: 'StephaneDucasse 7/5/2015 19:10'!
numberOfColumns
	"Answer the number of rows of the receiver."
	^ rows last size! !

!DhbMatrix methodsFor: 'accessing' stamp: 'StephaneDucasse 7/5/2015 19:10'!
numberOfRows
	"Answer the number of rows of the receiver."
	^ rows size! !

!DhbMatrix methodsFor: 'printing' stamp: 'StephaneDucasse 7/5/2015 19:24'!
printOn: aStream
	(rows isNil or: [rows first isNil])
		ifTrue: [ super printOn: aStream. 
					aStream nextPutAll:'(uninitialized)'. ^ self ].
	rows 
		do: [ :each | each printOn: aStream]
		separatedBy: [ aStream cr].
		  
		! !

!DhbMatrix methodsFor: 'private' stamp: 'StephaneDucasse 7/5/2015 19:26'!
privateTranspose
	^ self transpose ! !

!DhbMatrix methodsFor: 'double dispatching' stamp: 'StephaneDucasse 7/5/2015 19:05'!
productWithMatrix: aMatrix
	"Answers the product of aMatrix with the receiver (in this order)."
	^ self productWithMatrixFinal: aMatrix! !

!DhbMatrix methodsFor: 'double dispatching' stamp: 'StephaneDucasse 7/5/2015 19:05'!
productWithMatrixFinal: aMatrix
	"Answers the product of aMatrix with the receiver (in this order)."
	"speed optimized"
	|t|
	t :=self privateTranspose.
	^ DhbMatrix  rows: ( aMatrix rowsCollect: [ :row | t rowsCollect: [ :col | row * col]])! !

!DhbMatrix methodsFor: 'double dispatching' stamp: 'StephaneDucasse 7/5/2015 19:06'!
productWithTransposeMatrix: aMatrix
	"Answers the product of the receiver with the transpose of aMatrix(in this order)."
	^ DhbMatrix rows: (self rowsCollect: [ :row | aMatrix rowsCollect: [ :col | row * col]])! !

!DhbMatrix methodsFor: 'double dispatching' stamp: 'StephaneDucasse 7/5/2015 19:06'!
productWithVector: aVector
	"Answers the product of the receiver with aVector"
	^ self columnsCollect: [ :each | each * aVector ]! !

!DhbMatrix methodsFor: 'accessing' stamp: 'StephaneDucasse 7/5/2015 19:10'!
rowAt: anInteger
	"Answers the anInteger-th row of the receiver."
	^ rows at: anInteger! !

!DhbMatrix methodsFor: 'accessing' stamp: 'StephaneDucasse 7/5/2015 19:10'!
rowAt: aRowIndex columnAt: aColumnIndex
	"Answers the aRowIndex-th, aColumnIndex-th entry in the receiver."
	^ (rows at: aRowIndex) at: aColumnIndex! !

!DhbMatrix methodsFor: 'accessing' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
rowAt: aRowIndex columnAt: aColumnIndex put: aValue
	
	^(rows at: aRowIndex) at: aColumnIndex put: aValue! !

!DhbMatrix methodsFor: 'accessing' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
rows
	^rows! !

!DhbMatrix methodsFor: 'iterators' stamp: 'StephaneDucasse 7/5/2015 19:12'!
rowsCollect: aBlock
	"Perform the collect: operation on the rows of the receiver."
	^ rows collect: aBlock! !

!DhbMatrix methodsFor: 'iterators' stamp: 'StephaneDucasse 7/5/2015 19:13'!
rowsDo: aBlock
	"Perform the collect: operation on the rows of the receiver."
	^ rows do: aBlock! !

!DhbMatrix methodsFor: 'transformation' stamp: 'StephaneDucasse 7/5/2015 19:27'!
scaleBy: aNumber

	rows do: [ :each | each scaleBy: aNumber ]! !

!DhbMatrix methodsFor: 'private' stamp: 'StephaneDucasse 7/5/2015 19:26'!
species
	^ DhbMatrix ! !

!DhbMatrix methodsFor: 'private' stamp: 'StephaneDucasse 7/5/2015 19:27'!
split
	"Private - Answers an array of 4 matrices split from the receiver."
	| n m n1 m1 |
	n := self numberOfRows.
	m := self numberOfColumns.
	n1 := self largestPowerOf2SmallerThan: n.
	m1 := self largestPowerOf2SmallerThan: m.
	^ Array
		with: ( self class rows: ( ( 1 to: n1) asDhbVector collect: [ :k | ( rows at: k) copyFrom: 1 to: m1]))
		with:( self class rows: ( ( 1 to: n1) asDhbVector collect: [ :k | ( rows at: k) copyFrom: (m1 + 1) to: m]))
		with: ( self class rows: ( ( (n1 + 1) to: n) asDhbVector collect: [ :k | ( rows at: k) copyFrom: 1 to: m1]))
		with:( self class rows: ( ( (n1 + 1) to: n) asDhbVector collect: [ :k | ( rows at: k) copyFrom: (m1 + 1) to: m]))! !

!DhbMatrix methodsFor: 'operation' stamp: 'StephaneDucasse 7/5/2015 19:15'!
squared
	| transposed |
	transposed :=self privateTranspose.
	^ DhbSymmetricMatrix 
		new: transposed numberOfRows 
		function: [ :x :y|(transposed rowAt: x) * (transposed rowAt: y) ]
! !

!DhbMatrix methodsFor: 'private' stamp: 'StephaneDucasse 7/5/2015 19:16'!
strassenProductWithMatrix: aMatrix
	"Private"
	| matrixSplit selfSplit p1 p2 p3 p4 p5 p6 p7 |
	( self numberOfRows > 2 and: [ self numberOfColumns > 2])
		ifFalse:[ ^self class rows: ( aMatrix rowsCollect: [ :row | self columnsCollect: [ :col | row * col]])].
	selfSplit := self split.
	matrixSplit := aMatrix split.
	p1 := ( ( selfSplit at: 2) - ( selfSplit at: 4)) strassenProductWithMatrix: ( matrixSplit at: 1).
	p2 := ( selfSplit at: 4) strassenProductWithMatrix: ( ( matrixSplit at: 1) + ( matrixSplit at: 2)).
	p3 := ( selfSplit at: 1) strassenProductWithMatrix: ( ( matrixSplit at: 3) + ( matrixSplit at: 4)).
	p4 := ( ( selfSplit at: 3) - ( selfSplit at: 1)) strassenProductWithMatrix: ( matrixSplit at: 4).
	p5 := ( ( selfSplit at: 1) + ( selfSplit at: 4)) strassenProductWithMatrix: ( ( matrixSplit at: 1) + ( matrixSplit at: 4)).
	p6 := ( ( selfSplit at: 3) + ( selfSplit at: 4)) strassenProductWithMatrix: ( ( matrixSplit at: 2) - ( matrixSplit at: 4)).
	p7 := ( ( selfSplit at: 1) + ( selfSplit at: 2)) strassenProductWithMatrix: ( ( matrixSplit at: 1) - ( matrixSplit at: 3)).
	^self class join: ( Array
							with: ( p5 + p4 + p6 - p2)
							with: (p1 + p2)
							with: ( p3 + p4)
							with: ( p5 + p1 - p3 - p7)
							)! !

!DhbMatrix methodsFor: 'double dispatching' stamp: 'StephaneDucasse 7/5/2015 19:06'!
subtractWithRegularMatrix: aMatrix
	"Answers the difference between aMatrix and the receiver as a Matrix."
	| n |
	n := 0.
	^ DhbMatrix rows: ( aMatrix rowsCollect: [ :each | n := n + 1. each - ( self rowAt: n)])! !

!DhbMatrix methodsFor: 'double dispatching' stamp: 'StephaneDucasse 7/5/2015 19:07'!
subtractWithSymmetricMatrix: aMatrix
	"Answers the difference between aMatrix and the receiver."
	^ self subtractWithRegularMatrix: aMatrix ! !

!DhbMatrix methodsFor: 'operation' stamp: 'StephaneDucasse 7/5/2015 19:11'!
transpose
	"Answer a new matrix, transpose of the receiver."
	^ self class rows: ( self columnsCollect: [ :each | each])! !

!DhbMatrix methodsFor: 'double dispatching' stamp: 'StephaneDucasse 7/5/2015 19:07'!
transposeProductWithMatrix: aMatrix
	"Answers the product of the transpose of the receiver with aMatrix (in this order)."
	"speed optimized"
	|t|
	t :=aMatrix privateTranspose.
	^ DhbMatrix rows: (self columnsCollect: [ :row | t rowsCollect: [ :col | row * col]])! !

DhbMatrix subclass: #DhbSymmetricMatrix
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical-Math-Matrix'!

!DhbSymmetricMatrix commentStamp: 'StephaneDucasse 7/5/2015 21:07' prior: 0!
This class can be instantiated like DhbMatrix via #rows:, but the user has to make sure that this matrix is indeed symmetric as this is not internally checked for speed reasons.!

!DhbSymmetricMatrix class methodsFor: 'instance creation' stamp: 'wernerkassens 5/1/2015 11:59'!
identity: anInteger
		"Create an identity matrix of dimension anInteger"
	^self new initializeIdentity: anInteger! !

!DhbSymmetricMatrix class methodsFor: 'instance creation' stamp: 'wernerkassens 5/1/2015 11:59'!
join: anArrayOfMatrices
		"Inverse of the split operation"
	| rows n |
	rows := OrderedCollection new.
	n := 0.
	( anArrayOfMatrices at: 1) rowsDo:
		[ :each |
		  n := n + 1.
		  rows add: each, ( ( anArrayOfMatrices at: 3) columnAt: n).
		].
	n := 0.
	( anArrayOfMatrices at: 2) rowsDo:
		[ :each |
		  n := n + 1.
		  rows add: ( ( anArrayOfMatrices at: 3) rowAt: n), each.
		].
	^self rows: rows! !

!DhbSymmetricMatrix class methodsFor: 'information' stamp: 'wernerkassens 5/1/2015 12:00'!
lupCRLCriticalDimension
	^36! !

!DhbSymmetricMatrix class methodsFor: 'instance creation' stamp: 'wernerkassens 5/1/2015 10:12'!
new: dim function: aBlock
"Answer a new symmetric matrix with dimensions dim filled with 
aBlock value: RowposRespectivelyColpos value: ColposRespectivelyRowpos"
	|a aRow|
	a:=self new basicInitialize: dim.
	aRow := DhbVector new: dim.
	1 to: dim do:[:i|aRow at: i put: (aBlock value: i value:1)].
	a rows at: 1 put: aRow.
	2 to: dim do: [:rowPos|
		aRow :=DhbVector new: dim .
		1 to: rowPos -1 do:[:pos| aRow at: pos put: ((a rows at: pos) at: rowPos)].
		rowPos to: dim do:[:pos| aRow at: pos put: (aBlock value: rowPos value:pos)]. 
		(a rows) at: rowPos put: aRow ].    
	^a ! !

!DhbSymmetricMatrix methodsFor: 'operation' stamp: 'StephaneDucasse 7/5/2015 21:13'!
+ aMatrix
	"Answers the sum of the receiver with aMatrix."
	^ aMatrix addWithSymmetricMatrix: self! !

!DhbSymmetricMatrix methodsFor: 'operation' stamp: 'StephaneDucasse 7/5/2015 21:13'!
- aMatrix
	"Answers the difference between the receiver and aMatrix"
	^ aMatrix subtractWithSymmetricMatrix: self! !

!DhbSymmetricMatrix methodsFor: 'double dispatching' stamp: 'StephaneDucasse 7/5/2015 21:07'!
addWithSymmetricMatrix: aMatrix
	^self class 
		new: aMatrix numberOfRows 
		function: [ :x :y|((rows at:x)at:y)+((aMatrix rows at:x)at:y) ]
! !

!DhbSymmetricMatrix methodsFor: 'transformation' stamp: 'WernerKassens 4/8/2014 22:36'!
asSymmetricMatrix
"simple speed up"
^self! !

!DhbSymmetricMatrix methodsFor: 'information' stamp: 'StephaneDucasse 7/5/2015 21:08'!
atColumn: aColumnIndex put: aCollection
	self shouldNotImplement ! !

!DhbSymmetricMatrix methodsFor: 'information' stamp: 'StephaneDucasse 7/5/2015 21:08'!
atRow: aRowIndex put: aCollection
	self shouldNotImplement ! !

!DhbSymmetricMatrix methodsFor: 'initialization' stamp: 'StephaneDucasse 7/5/2015 21:11'!
basicInitialize: anInteger
	"private"
	rows := DhbVector new: anInteger.! !

!DhbSymmetricMatrix methodsFor: 'initialization' stamp: 'StephaneDucasse 7/5/2015 21:11'!
clear
	
	rows do: [ :each | each atAllPut: 0].! !

!DhbSymmetricMatrix methodsFor: 'information' stamp: 'StephaneDucasse 7/5/2015 21:11'!
columnAt: anInteger
	^ rows at: anInteger   ! !

!DhbSymmetricMatrix methodsFor: 'iterators' stamp: 'StephaneDucasse 7/5/2015 21:12'!
columnsCollect: aBlock
	^ rows collect: aBlock ! !

!DhbSymmetricMatrix methodsFor: 'iterators' stamp: 'StephaneDucasse 7/5/2015 21:12'!
columnsDo: aBlock
	^ rows do: aBlock ! !

!DhbSymmetricMatrix methodsFor: 'operation' stamp: 'StephaneDucasse 7/5/2015 21:13'!
crlInverse
	"corrected sequence of calcs; s1in matrices at:2 has to be calced first (split returns matrices in this order: b, d, c).
and taken care of degenerate cases (which is unnecessary, if this method is not called outside of inverse)"
	| matrices b1 cb1ct cb1 |
	rows size = 1
		ifTrue: [ ^self inverse1By1].
	rows size = 2
		ifTrue: [ ^self inverse2By2].
	matrices := self split.
	b1 := (matrices at: 1) inverse.
	cb1 := (matrices at: 3) * b1.
	cb1ct := (cb1 productWithTransposeMatrix: (matrices at: 3))asSymmetricMatrix .
	matrices at: 2 put: ((matrices at: 2) - cb1ct ) inverse.
	matrices at: 3 put: (matrices at: 2) * cb1.
	matrices at: 1 put: ( b1 + (cb1 transposeProductWithMatrix: (matrices at: 3))asSymmetricMatrix).
	(matrices at: 3) negate.
	^ self class join: matrices! !

!DhbSymmetricMatrix methodsFor: 'initialization' stamp: 'StephaneDucasse 7/5/2015 21:12'!
initializeIdentity: anInteger
	"Build components for an identity matrix."
	rows := ( 1 to: anInteger) asDhbVector collect: [ :n | (DhbVector new: anInteger) atAllPut: 0; at: n put: 1; yourself].! !

!DhbSymmetricMatrix methodsFor: 'operation' stamp: 'StephaneDucasse 7/5/2015 21:13'!
inverse
	"Answer the inverse of the receiver"

	^(rows size < self class lupCRLCriticalDimension or: [lupDecomposition notNil]) 
			ifTrue: [ self lupInverse ]
			ifFalse: [ self crlInverse ]! !

!DhbSymmetricMatrix methodsFor: 'operation' stamp: 'StephaneDucasse 7/5/2015 21:13'!
inverse1By1
	"Private - Answer the inverse of the receiver when it is a 1x1 matrix (no check is made)."
	^ self class rows: ( DhbVector with: ( DhbVector with: ( 1/ ((rows at: 1) at: 1))))! !

!DhbSymmetricMatrix methodsFor: 'operation' stamp: 'StephaneDucasse 7/5/2015 21:14'!
inverse2By2
	"Private - Answer the inverse of the receiver when it is a 2x2 matrix (no check is made)."
	| line1 line2 |
	line1 := DhbVector with: ((rows at: 2) at: 2) with: ((rows at: 1) at: 2) negated.
	line2 := DhbVector with: ((rows at: 1) at: 2) negated with: ((rows at: 1) at: 1).
	^self class rows: ( DhbVector with: line1 with: line2) * (1 / (((rows at: 1) at: 1) * ((rows at: 2) at: 2) - ((rows at: 1) at: 2) squared))! !

!DhbSymmetricMatrix methodsFor: 'operation' stamp: 'StephaneDucasse 7/5/2015 21:14'!
inversePureCRL
	"Answer the inverse of the receiver."
	| matrices b1 cb1 cb1ct |
	[rows size = 1
		ifTrue: [ ^self inverse1By1]]on: ZeroDivide do: [:e| SingularMatrixError new signal].
	rows size = 2
		ifTrue: [ ^ self inverse2By2].
	matrices := self split.
	b1 := (matrices at: 1) inversePureCRL.
	cb1 := (matrices at: 3) * b1.
	cb1ct := (cb1 productWithTransposeMatrix: ( matrices at: 3)) asSymmetricMatrix.
	matrices at: 2 put: ((matrices at: 2) - cb1ct) inversePureCRL.
	matrices at: 3 put: (matrices at: 2) * cb1.
	matrices at: 1 put: (b1 + ( cb1 transposeProductWithMatrix: ( matrices at: 3))asSymmetricMatrix).
	(matrices at: 3) negate.
	^ self class join: matrices! !

!DhbSymmetricMatrix methodsFor: 'operation' stamp: 'StephaneDucasse 7/5/2015 21:14'!
inversePureLUP
	"Answer the inverse of the receiver."
	lupDecomposition := nil.
	self lupDecomposition inverseMatrixComponents 
		ifNil: [ SingularMatrixError new signal ] 
		ifNotNilDo: [ :i| ^self class rows: i ].
! !

!DhbSymmetricMatrix methodsFor: 'testing' stamp: 'StephaneDucasse 7/5/2015 21:15'!
isSquare
	"Answers true because a symmetric matrix is square."
	^true! !

!DhbSymmetricMatrix methodsFor: 'testing' stamp: 'wernerkassens 5/1/2015 11:57'!
isSymmetric
		"Answers true because the receiver is a symmetric matrix"
	^true! !

!DhbSymmetricMatrix methodsFor: 'information' stamp: 'StephaneDucasse 7/5/2015 21:08'!
numberOfColumns
	"Answer the number of columns of the receiver."
	^rows size! !

!DhbSymmetricMatrix methodsFor: 'private ' stamp: 'StephaneDucasse 7/5/2015 21:15'!
privateTranspose 
	^ self ! !

!DhbSymmetricMatrix methodsFor: 'transformation' stamp: 'wernerkassens 5/1/2015 11:58'!
split
	"Private - 
	Answers an array of 3 matrices split from the receiver"

	| n |
	n := self largestPowerOf2SmallerThan: rows size.
	^Array
		with: (self class rows: ((1 to: n) asDhbVector collect: [:k | (rows at: k)
						copyFrom: 1 to: n]))
		with: (self class rows: ((n + 1 to: rows size) asDhbVector collect: [:k | (rows at: k)
						copyFrom: n + 1 to: rows size]))
		with: (self class superclass rows: ((n + 1 to: rows size) asDhbVector collect: [:k | (rows at: k)
						copyFrom: 1 to: n]))! !

!DhbSymmetricMatrix methodsFor: 'double dispatching' stamp: 'StephaneDucasse 7/5/2015 21:07'!
subtractWithSymmetricMatrix: aMatrix
	^ self class 
		new: aMatrix numberOfRows 
		function: [ :x :y|((aMatrix rows at:x)at:y)-(( rows at:x)at:y) ]! !

!DhbSymmetricMatrix methodsFor: 'information' stamp: 'StephaneDucasse 7/5/2015 21:08'!
transpose
	"Answer a new matrix, transpose of the receiver. optimized"
	^self deepCopy! !

Object subclass: #DhbMinimizingPoint
	instanceVariableNames: 'value position'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

DhbMinimizingPoint subclass: #DhbMaximizingPoint
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbMaximizingPoint methodsFor: 'information' stamp: 'StephaneDucasse 7/12/2015 11:12'!
better: anOptimizingPoint
	^ value >= anOptimizingPoint value! !

!DhbMaximizingPoint methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
betterThan: anOptimizingPoint
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 04-Jan-00 "
	^value > anOptimizingPoint value! !

!DhbMinimizingPoint class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
new: aVector value: aNumber
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 04-Jan-00 "
	^self new vector: aVector; value: aNumber; yourself! !

!DhbMinimizingPoint class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
vector: aVector function: aFunction 
	"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 04-Jan-00 "

	^self new: aVector value: (aFunction value: aVector)! !

!DhbMinimizingPoint methodsFor: 'comparing' stamp: 'wk 4/14/2012 01:45'!
=aMPoint
"only for testing purposes"
^(value = aMPoint value)& (position = aMPoint position)! !

!DhbMinimizingPoint methodsFor: 'information' stamp: 'StephaneDucasse 7/12/2015 11:11'!
better: anOptimizingPoint
	^ value <= anOptimizingPoint value! !

!DhbMinimizingPoint methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
betterThan: anOptimizingPoint
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 04-Jan-00 "
	^value < anOptimizingPoint value! !

!DhbMinimizingPoint methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
position
	"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 04-Jan-00 "

	^position! !

!DhbMinimizingPoint methodsFor: 'display' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
printOn: aStream 
	"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 05-Jan-00 "

	position printOn: aStream.
	aStream
		nextPut: $:;
		space.
	value printOn: aStream! !

!DhbMinimizingPoint methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
value
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 04-Jan-00 "
	^value! !

!DhbMinimizingPoint methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
value: aNumber
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 04-Jan-00 "
	value := aNumber.! !

!DhbMinimizingPoint methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
vector: aVector 
	"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 04-Jan-00 "

	position := aVector! !

Object subclass: #DhbMitchellMooreGenerator
	instanceVariableNames: 'randoms lowIndex highIndex'
	classVariableNames: 'UniqueInstance'
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbMitchellMooreGenerator class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
constants: anArray lowIndex: anInteger
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 1/11/00 "
	^super new initialize: anArray lowIndex: anInteger! !

!DhbMitchellMooreGenerator class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
default
	"Private- 
	 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 1/11/00 "

	| congruentialGenerator |
	congruentialGenerator := DhbCongruentialRandomNumberGenerator new.
	^self generateSeeds: congruentialGenerator! !

!DhbMitchellMooreGenerator class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
generateSeeds: congruentialGenerator
	"Private- "
	^self constants: ((1 to: 55) collect: [:n | congruentialGenerator floatValue])
		lowIndex: 24! !

!DhbMitchellMooreGenerator class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
new
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 1/11/00 "
	UniqueInstance isNil
		ifTrue: [ UniqueInstance := self default].
	^UniqueInstance! !

!DhbMitchellMooreGenerator class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
reset: anInteger
	"Reset the unique instance used for the default series"
	UniqueInstance := self seed: anInteger.! !

!DhbMitchellMooreGenerator class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
seed: anInteger
	"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 1/18/00 "

	| congruentialGenerator |
	congruentialGenerator := DhbCongruentialRandomNumberGenerator seed: anInteger.
	^self generateSeeds: congruentialGenerator! !

!DhbMitchellMooreGenerator methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
floatValue
	"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 1/11/00 "

	| x |
	x := (randoms at: lowIndex) + (randoms at: highIndex).
	x < 1.0 ifFalse: [x := x - 1.0].
	randoms at: highIndex put: x.
	highIndex := highIndex + 1.
	highIndex > randoms size ifTrue: [highIndex := 1].
	lowIndex := lowIndex + 1.
	lowIndex > randoms size ifTrue: [lowIndex := 1].
	^x! !

!DhbMitchellMooreGenerator methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialize: anArray lowIndex: anInteger
	"Private -
		 (c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 1/11/00 "

	randoms := anArray.
	lowIndex := anInteger.
	highIndex := randoms size.
	^self! !

!DhbMitchellMooreGenerator methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
integerValue: anInteger
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 1/11/00 "
	^( self floatValue * anInteger) truncated! !

Object subclass: #DhbPolynomial
	instanceVariableNames: 'coefficients'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbPolynomial commentStamp: '<historical>' prior: 0!
A DhbPolynomial represents a single variable polynomial function as an array of coefficients.

The constructor coefficients: anArray expects coefficients in increasing degree, that is constant term first. So, for example, to represent X^2 + 2X, we want to create:

DhbPolynomial coefficients: #(0 2 1)

The printOn: method similarly displays the polynomial from lowest nonzero degree to highest. 

DhbPolynomial coefficients: #(0 2 1)   2 X +  X^2 

!

DhbPolynomial subclass: #DhbEstimatedPolynomial
	instanceVariableNames: 'errorMatrix'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbEstimatedPolynomial methodsFor: 'information' stamp: 'WernerKassens 4/15/2014 09:57'!
error: aNumber
		"Compute the error on the value of the receiver for argument aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 18/3/99 "
	| errorVector term nextTerm |
	nextTerm := 1.
	errorVector := ( coefficients collect: [ :each | term := nextTerm. nextTerm := aNumber * nextTerm. term]) asDhbVector.
	^( errorVector * errorMatrix * errorVector) sqrt! !

!DhbEstimatedPolynomial methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
errorMatrix
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 21/5/99 "
	^errorMatrix! !

!DhbEstimatedPolynomial methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
errorMatrix: aMatrix
		"Defines the error matrix of the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 18/3/99 "
	errorMatrix := aMatrix.! !

!DhbEstimatedPolynomial methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
valueAndError: aNumber
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 20/5/99 "
	^Array with: ( self value: aNumber) with: ( self error: aNumber)! !

!DhbPolynomial class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
coefficients: anArray
		" Creates a new instance with given coefficients
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 4/1/99 "
	^self new initialize: anArray reverse! !

!DhbPolynomial methodsFor: 'operation' stamp: 'StephaneDucasse 3/1/2016 21:27'!
* aNumberOrPolynomial

	^aNumberOrPolynomial timesPolynomial: self! !

!DhbPolynomial methodsFor: 'operation' stamp: 'StephaneDucasse 3/1/2016 21:27'!
+ aNumberOrPolynomial
		
	^aNumberOrPolynomial addPolynomial: self! !

!DhbPolynomial methodsFor: 'operation' stamp: 'StephaneDucasse 3/1/2016 21:29'!
- aNumberOrPolynomial

	^aNumberOrPolynomial subtractToPolynomial: self! !

!DhbPolynomial methodsFor: 'operation' stamp: 'StephaneDucasse 3/1/2016 21:29'!
/ aNumberOrPolynomial
	
	^aNumberOrPolynomial dividingPolynomial: self! !

!DhbPolynomial methodsFor: 'comparing' stamp: 'wernerkassens 11/7/2014 12:39'!
= aNumberOrPolynomial 
	^(aNumberOrPolynomial class = self class) and: [self coefficients = aNumberOrPolynomial coefficients]! !

!DhbPolynomial methodsFor: 'double dispatching' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
adaptToNumber: rcvr andSend: selector
	^(self class coefficients: (Array with: rcvr) ) perform: selector with: self! !

!DhbPolynomial methodsFor: 'double dispatching' stamp: 'StephaneDucasse 3/1/2016 21:29'!
addNumber: aNumber
	
	| newCoefficients |
	newCoefficients := coefficients reverse.
	newCoefficients at: 1 put: newCoefficients first + aNumber.
	^self class coefficients: newCoefficients! !

!DhbPolynomial methodsFor: 'double dispatching' stamp: 'StephaneDucasse 3/1/2016 21:29'!
addPolynomial: aPolynomial
	
	^self class coefficients: ( ( 0 to: (self degree max: aPolynomial degree)) collect: [ :n | ( aPolynomial at: n) + ( self at: n)])! !

!DhbPolynomial methodsFor: 'information' stamp: 'StephaneDucasse 3/1/2016 21:29'!
at: anInteger
	"Answers the coefficient of order anInteger."
	^anInteger < coefficients size
		ifTrue: [ coefficients at: ( coefficients size - anInteger)]
		ifFalse:[ 0]! !

!DhbPolynomial methodsFor: 'information' stamp: 'StephaneDucasse 3/1/2016 21:29'!
coefficients

	^coefficients reverse! !

!DhbPolynomial methodsFor: 'creation' stamp: 'StephaneDucasse 3/1/2016 21:26'!
deflatedAt: aNumber
	"Answers a new polynomial quotient of the receiver with polynomial (X-aNumber)"
	| remainder next newCoefficients |
	remainder := 0.
	newCoefficients := coefficients collect:
						[ :each |
						  next := remainder. 
						  remainder := remainder * aNumber + each.
						  next].
	^self class coefficients: ( newCoefficients copyFrom: 2 to: newCoefficients size) reverse! !

!DhbPolynomial methodsFor: 'information' stamp: 'StephaneDucasse 3/1/2016 21:30'!
degree

	^ coefficients size - 1! !

!DhbPolynomial methodsFor: 'creation' stamp: 'StephaneDucasse 3/1/2016 21:26'!
derivative
	"Answer a new polynomial, derivative of the receiver."
	| n |
	n := coefficients size.
	^self class coefficients: ( ( coefficients collect: [ :each | n := n - 1. each * n]) reverse copyFrom: 2 to: coefficients size)! !

!DhbPolynomial methodsFor: 'double dispatching' stamp: 'StephaneDucasse 3/1/2016 21:30'!
dividingPolynomial: aPolynomial
		
	^( self dividingPolynomialWithRemainder: aPolynomial) first! !

!DhbPolynomial methodsFor: 'double dispatching' stamp: 'StephaneDucasse 3/1/2016 21:30'!
dividingPolynomialWithRemainder: aPolynomial
	
	| remainderCoefficients quotientCoefficients n m norm quotientDegree |
	n := self degree.
	m := aPolynomial degree.
	quotientDegree := m - n.
	quotientDegree < 0
		"this is a bugfix: variable sized instance : Behavior basicNew"
		ifTrue: [ ^Array with: ( self class coefficients: #(0)) with: aPolynomial].
	quotientCoefficients := Array new: quotientDegree + 1.
	remainderCoefficients := ( 0 to: m) collect: [ :k | aPolynomial at: k].
	norm := 1 / coefficients first.
	quotientDegree to: 0 by: -1
		do: [ :k | | x |
			  x := ( remainderCoefficients at: n + k + 1) * norm.
			  quotientCoefficients at: (quotientDegree + 1 - k) put: x.
			  (n + k - 1) to: k by: -1
				do: [ :j | 
				remainderCoefficients at: j + 1 put: 
							( ( remainderCoefficients at: j + 1) - ( x * (self at: j - k)))
				].
			].
	^Array with: ( self class coefficients: quotientCoefficients reverse)
		   with: ( self class coefficients: ( remainderCoefficients copyFrom: 1 to: n))! !

!DhbPolynomial methodsFor: 'comparing' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
hash
	^coefficients hash! !

!DhbPolynomial methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialize
	"initialize an empty polynomial as the constant zero"
	super initialize.
	coefficients := #( 0 ).
	^ self! !

!DhbPolynomial methodsFor: 'initialization' stamp: 'StephaneDucasse 3/1/2016 21:30'!
initialize: anArray
	
	coefficients := anArray.
	^self! !

!DhbPolynomial methodsFor: 'creation' stamp: 'StephaneDucasse 3/1/2016 21:26'!
integral
	"Answer a new polynomial, integral of the receiver with value 0 at x=0."
	^self integral: 0! !

!DhbPolynomial methodsFor: 'creation' stamp: 'StephaneDucasse 3/1/2016 21:27'!
integral: aValue
	"Answer a new polynomial, integral of the receiver with given value at x=0."
	| n |
	n := coefficients size + 1.
	^self class coefficients: ( ( coefficients collect: [ :each | n := n - 1. each / n]) copyWith: aValue) reverse! !

!DhbPolynomial methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
isZero
	^ self coefficients allSatisfy: [:ea | ea = 0].! !

!DhbPolynomial methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
negated
	^ self * -1.! !

!DhbPolynomial methodsFor: 'printing' stamp: 'StephaneDucasse 3/1/2016 21:27'!
printOn: aStream 
	"Append to aStream a written representation of the receiver."

	| n firstNonZeroCoefficientPrinted |
	n := 0.
	firstNonZeroCoefficientPrinted := false.
	coefficients
		reverseDo: 
			[:each | 
			each = 0
				ifFalse: 
					[firstNonZeroCoefficientPrinted
						ifTrue: 
							[aStream space.
							((each respondsTo: #< ) and: [each < 0]) ifFalse: [aStream nextPut: $+].
							aStream space]
						ifFalse: [firstNonZeroCoefficientPrinted := true].
					(each = 1 and: [n > 0])
						ifFalse: [each printOn: aStream].
					n > 0
						ifTrue: 
							[aStream nextPutAll: ' X'.
							n > 1
								ifTrue: 
									[aStream nextPut: $^.
									n printOn: aStream]]].
			n := n + 1].
		
	"add a quick check, print a zero if the polynomial has all zero coefficients
	Daniel Uber 11 May 2012"
	firstNonZeroCoefficientPrinted ifFalse: [0 printOn: aStream].! !

!DhbPolynomial methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
reciprocal
	^ (DhbPolynomial coefficients: #(1)) / self! !

!DhbPolynomial methodsFor: 'information' stamp: 'StephaneDucasse 3/1/2016 21:30'!
roots
	
	^self roots: DhbFloatingPointMachine new defaultNumericalPrecision! !

!DhbPolynomial methodsFor: 'information' stamp: 'StephaneDucasse 3/1/2016 21:30'!
roots: aNumber

	| pol roots x rootFinder |
	rootFinder := DhbNewtonZeroFinder new.
	rootFinder desiredPrecision: aNumber.
	pol := self class coefficients: ( coefficients reverse collect: [ :each | each asFloat]).
	roots := OrderedCollection new: self degree.
	[ rootFinder setFunction: pol; setDerivative: pol derivative.
	  x := rootFinder evaluate.
	  rootFinder hasConverged
		] whileTrue: [ roots add: x. 
					   pol := pol deflatedAt: x. 
					   pol degree > 0
						 ifFalse: [ ^roots].
					 ].
	^roots! !

!DhbPolynomial methodsFor: 'double dispatching' stamp: 'StephaneDucasse 3/1/2016 21:30'!
subtractToPolynomial: aPolynomial

	^self class coefficients: ( ( 0 to: (self degree max: aPolynomial degree)) collect: [ :n | ( aPolynomial at: n) - ( self at: n)])! !

!DhbPolynomial methodsFor: 'double dispatching' stamp: 'StephaneDucasse 3/1/2016 21:30'!
timesNumber: aNumber
	
	^self class coefficients: ( coefficients reverse collect: [ :each | each * aNumber])! !

!DhbPolynomial methodsFor: 'double dispatching' stamp: 'StephaneDucasse 3/1/2016 21:30'!
timesPolynomial: aPolynomial
	
	| productCoefficients degree|
	degree := aPolynomial degree + self degree.
	productCoefficients := ( degree to: 0 by: -1)
			collect:[ :n | | sum |
					  sum := 0.
					  0 to: (degree - n)
						do: [ :k | sum := (self at: k) * (aPolynomial at: ( degree - n - k)) + sum].
					  sum
					].
	^self class coefficients: productCoefficients! !

!DhbPolynomial methodsFor: 'information' stamp: 'StephaneDucasse 3/1/2016 21:30'!
value: aNumber
	"Answer the value of the polynomial for the specified variable value."
	^coefficients inject: 0 into: [ :sum :each | sum * aNumber + each]! !

Object subclass: #DhbPolynomialLeastSquareFit
	instanceVariableNames: 'pointCollection degreePlusOne'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbPolynomialLeastSquareFit class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
new: anInteger
		"Create a new instance of the receiver with given degree.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 18/3/99 "
	^super new initialize: anInteger! !

!DhbPolynomialLeastSquareFit class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
new: anInteger on: aCollectionOfPoints
		"Create a new instance of the receiver with given degree and points.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 18/3/99 "
	^super new initialize: anInteger on: aCollectionOfPoints! !

!DhbPolynomialLeastSquareFit methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
accumulate: aWeightedPoint into: aVectorOfVectors and: aVector
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 18/3/99 "
	| t p powers |
	p := 1.0.
	powers := aVector collect: [ :each | t := p. p := p * aWeightedPoint xValue. t].
	aVector accumulate: powers * ( aWeightedPoint yValue * aWeightedPoint weight).
	1 to: aVector size do:
		[ :k |
		  ( aVectorOfVectors at: k) accumulate: powers * ( ( powers at: k) * aWeightedPoint weight).
		].! !

!DhbPolynomialLeastSquareFit methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
add: aWeightedPoint
		"Add a point to the collection of points.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 18/3/99 "
	^pointCollection add: aWeightedPoint! !

!DhbPolynomialLeastSquareFit methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
computeEquations
		"Private - Answer a pair Matrix/Vector defining the system of equations
		 to solve the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 18/3/99 "
	| rows vector |
	vector := ( DhbVector new: degreePlusOne) atAllPut: 0 ; yourself.
	rows := ( 1 to: degreePlusOne) collect: [ :k | ( DhbVector new: degreePlusOne) atAllPut: 0 ; yourself].
	pointCollection do:
		[ :each | self accumulate: each into: rows and: vector].
	^Array with: ( DhbSymmetricMatrix rows: rows) with: vector! !

!DhbPolynomialLeastSquareFit methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
evaluate
		"Perform the least square fit and answers the fitted polynomial.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 18/3/99 "
	| system errorMatrix |
	system := self computeEquations.
	errorMatrix := ( system at: 1) inverse.
	^( DhbEstimatedPolynomial coefficients: errorMatrix * (system at: 2))
			errorMatrix: errorMatrix;
			yourself! !

!DhbPolynomialLeastSquareFit methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialize: anInteger
		"Private - Create an empty point collection for the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 18/3/99 "
	^self initialize: anInteger on: OrderedCollection new! !

!DhbPolynomialLeastSquareFit methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialize: anInteger on: aCollectionOfPoints
		"Private - Defines the collection of points for the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 18/3/99 "
	pointCollection := aCollectionOfPoints.
	degreePlusOne := anInteger + 1.
	^self! !

Object subclass: #DhbProbabilityDistributionFunction
	instanceVariableNames: 'probabilityDensity'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbProbabilityDistributionFunction class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
density: aProbabilityDensity
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/4/99 "
	^self new initialize: aProbabilityDensity! !

!DhbProbabilityDistributionFunction methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialize: aProbabilityDensity
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/4/99 "
	probabilityDensity := aProbabilityDensity.
	^self! !

!DhbProbabilityDistributionFunction methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
value: aNumber
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/4/99 "
	^probabilityDensity distributionValue: aNumber! !

Object subclass: #DhbProjectedOneVariableFunction
	instanceVariableNames: 'index function argument'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbProjectedOneVariableFunction class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
function: aVectorFunction
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	^super new initialize: aVectorFunction! !

!DhbProjectedOneVariableFunction methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
argumentWith: aNumber
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	^argument at: index put: aNumber; yourself! !

!DhbProjectedOneVariableFunction methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
bumpIndex
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	index isNil
		ifTrue: [ index := 1]
		ifFalse:[ index := index + 1.
				  index > argument size
					ifTrue: [ index := 1].
				].! !

!DhbProjectedOneVariableFunction methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
index
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	index isNil
		ifTrue: [ index := 1].
	^index! !

!DhbProjectedOneVariableFunction methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialize: aFunction
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	function := aFunction.
	^self! !

!DhbProjectedOneVariableFunction methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
setArgument: anArrayOrVector
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	argument := anArrayOrVector copy.! !

!DhbProjectedOneVariableFunction methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
setIndex: anInteger
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	index := anInteger.! !

!DhbProjectedOneVariableFunction methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
value: aNumber
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	^function value: ( self argumentWith: aNumber)! !

DhbProjectedOneVariableFunction subclass: #DhbVectorProjectedFunction
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbVectorProjectedFunction methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
argumentWith: aNumber
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	^aNumber * self direction + self origin! !

!DhbVectorProjectedFunction methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
direction
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	^index! !

!DhbVectorProjectedFunction methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
direction: aVector
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	index := aVector.! !

!DhbVectorProjectedFunction methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
origin
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	^argument! !

!DhbVectorProjectedFunction methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
origin: aVector
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	argument := aVector.! !

!DhbVectorProjectedFunction methodsFor: 'display' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
printOn: aStream
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/22/00 "
	self origin printOn: aStream.
	aStream nextPutAll: ' ('.
	self direction printOn: aStream.
	aStream nextPut: $).! !

Object subclass: #DhbScaledProbabilityDensityFunction
	instanceVariableNames: 'probabilityDensityFunction count binWidth'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbScaledProbabilityDensityFunction class methodsFor: 'creation' stamp: 'WernerKassens 4/6/2014 13:05'!
histogram: aHistogram against: aProbabilityDensityFunction 
	"Create a new instance of the receiver with given probability density function and histogram.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 3/3/99 "
		^self new 
		initialize: aProbabilityDensityFunction
		binWidth: aHistogram binWidth
		count: aHistogram totalCount! !

!DhbScaledProbabilityDensityFunction class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
histogram: aHistogram distributionClass: aProbabilityDensityFunctionClass 
	"Create a new instance of the receiver with given probability density function and histogram. 
	(c) Copyrights Didier BESSET, 1999, all rights reserved. 
	Initial code: 3/3/99"

	| dp |
	^(dp := aProbabilityDensityFunctionClass fromHistogram: aHistogram) isNil
		ifTrue: [nil]
		ifFalse: [self histogram: aHistogram against: dp]! !

!DhbScaledProbabilityDensityFunction methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
changeParametersBy: aVector
		"Modify the parameters of the receiver by aVector.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/3/99 "
	count := count + aVector last.
	probabilityDensityFunction changeParametersBy: aVector.! !

!DhbScaledProbabilityDensityFunction methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
distributionFunction
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/4/99 "
	^probabilityDensityFunction distributionFunction! !

!DhbScaledProbabilityDensityFunction methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialize: aProbabilityDensityFunction binWidth: aNumber count: anInteger 
	"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 3/3/99 "

	probabilityDensityFunction := aProbabilityDensityFunction.
	binWidth := aNumber.
	count := anInteger.
	^self! !

!DhbScaledProbabilityDensityFunction methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
parameters
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/3/99 "
	^probabilityDensityFunction parameters copyWith: count! !

!DhbScaledProbabilityDensityFunction methodsFor: 'display' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
printOn: aStream
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/4/99 "
	super printOn: aStream.
	aStream nextPut: $[;
			nextPutAll: probabilityDensityFunction class distributionName;
			nextPut: $].! !

!DhbScaledProbabilityDensityFunction methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
setCount: aNumber
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 12/3/99 "
	count := aNumber.! !

!DhbScaledProbabilityDensityFunction methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
value: aNumber 
	"
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 3/3/99 "

	^(probabilityDensityFunction value: aNumber) * binWidth * count! !

!DhbScaledProbabilityDensityFunction methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
valueAndGradient: aNumber 
	"Answers an Array containing the value of the receiver at aNumber
		 and the gradient of the receiver's respective to the receiver's
		 parameters evaluated at aNumber.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 11/3/99 "

	| g temp |
	g := probabilityDensityFunction valueAndGradient: aNumber.
	temp := binWidth * count.
	^Array with: g first * temp
		   with: ( (g last collect: [:each | each * temp]) copyWith: g first * binWidth)! !

Object subclass: #DhbSeriesTermServer
	instanceVariableNames: 'x lastTerm'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

DhbSeriesTermServer subclass: #DhbIncompleteBetaFractionTermServer
	instanceVariableNames: 'alpha1 alpha2'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbIncompleteBetaFractionTermServer methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialTerm
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/3/99 "
	^1! !

!DhbIncompleteBetaFractionTermServer methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
setParameter: aNumber1 second: aNumber2 
	"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/3/99 "

	alpha1 := aNumber1.
	alpha2 := aNumber2! !

!DhbIncompleteBetaFractionTermServer methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
termsAt: anInteger 
	"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/3/99 "

	| n n2 |
	n := anInteger // 2.
	n2 := 2 * n.
	^Array with: ( n2 < anInteger 
		ifTrue: 
			[x negated * (alpha1 + n) * (alpha1 + alpha2 + n) 
				/ ((alpha1 + n2) * (alpha1 + 1 + n2))]
		ifFalse: [x * n * (alpha2 - n) / ((alpha1 + n2) * (alpha1 - 1 + n2))])
			with: 1! !

DhbSeriesTermServer subclass: #DhbIncompleteGammaFractionTermServer
	instanceVariableNames: 'alpha'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbIncompleteGammaFractionTermServer methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialTerm
	"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/3/99 "

	lastTerm := x - alpha + 1.
	^lastTerm! !

!DhbIncompleteGammaFractionTermServer methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
setParameter: aNumber 
	"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/3/99 "

	alpha := aNumber asFloat! !

!DhbIncompleteGammaFractionTermServer methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
termsAt: anInteger 
	"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/3/99 "
	lastTerm := lastTerm + 2.
	^Array with: (alpha - anInteger) * anInteger with: lastTerm! !

DhbSeriesTermServer subclass: #DhbIncompleteGammaSeriesTermServer
	instanceVariableNames: 'alpha sum'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbIncompleteGammaSeriesTermServer methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialTerm
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/3/99 "
	lastTerm := 1 / alpha.
	sum := alpha.
	^lastTerm! !

!DhbIncompleteGammaSeriesTermServer methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
setParameter: aNumber 
	"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/3/99 "

	alpha := aNumber asFloat! !

!DhbIncompleteGammaSeriesTermServer methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
termAt: anInteger
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/3/99 "
	sum := sum + 1.
	lastTerm := lastTerm * x / sum.
	^lastTerm! !

!DhbSeriesTermServer methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
setArgument: aNumber
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/3/99 "
	x := aNumber asFloat.! !

Object subclass: #DhbVectorAccumulator
	instanceVariableNames: 'count average'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbVectorAccumulator commentStamp: 'StephaneDucasse 7/5/2015 19:46' prior: 0!
I'm a simple object summing vectors.!

DhbVectorAccumulator subclass: #DhbCovarianceAccumulator
	instanceVariableNames: 'covariance'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!DhbCovarianceAccumulator methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
accumulate: anArray
		"Accumulate anArray into the receiver.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 8/3/99 "
	| delta count1 r|
	count1 := count.
	delta := super accumulate: anArray.
	r := count1 / count.
	1 to: delta size
		do: [ :n |
			  1 to: n do:
				  [ :m |
					( covariance at: n) at: m put: ( count1 * ( delta at: n) * ( delta at: m) + ( r * ( ( covariance at: n) at: m))).
				  ].
			].! !

!DhbCovarianceAccumulator methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
covarianceMatrix
		"Answer a matrix containing the covariance of the accumulated data.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 8/3/99 "
	| rows n |
	n := 0.
	rows := covariance collect:
				[ :row | n := n + 1. row, ( ( ( n + 1) to: covariance size) collect: [ :m | ( covariance at: m) at: n ])].
	^DhbSymmetricMatrix rows: rows! !

!DhbCovarianceAccumulator methodsFor: 'initialization' stamp: 'WernerKassens 4/15/2014 09:57'!
initialize: anInteger
		"Private - Initialize the receiver to accumulate vectors of dimension anInteger.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 8/3/99 "
	covariance := ( ( 1 to: anInteger) collect: [ :n | DhbVector new: n]) asDhbVector.
	^super initialize: anInteger! !

!DhbCovarianceAccumulator methodsFor: 'transformation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
reset
		"Set all accumulators to zero.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 8/3/99 "
	super reset.
	covariance do: [ :each | each atAllPut: 0].! !

!DhbVectorAccumulator class methodsFor: 'creation' stamp: 'StephaneDucasse 7/5/2015 19:45'!
new: anInteger
	^ self basicNew initialize: anInteger! !

!DhbVectorAccumulator methodsFor: 'transformation' stamp: 'StephaneDucasse 7/5/2015 19:44'!
accumulate: aVectorOrArray

	| delta |
	count := count + 1.
	delta := average - aVectorOrArray asDhbVector scaleBy: 1 / count.
	average accumulateNegated: delta.
	^ delta! !

!DhbVectorAccumulator methodsFor: 'operations' stamp: 'StephaneDucasse 7/5/2015 19:43'!
average
	
	^average! !

!DhbVectorAccumulator methodsFor: 'operations' stamp: 'StephaneDucasse 7/5/2015 19:43'!
count
	
	^count! !

!DhbVectorAccumulator methodsFor: 'initialization' stamp: 'StephaneDucasse 7/5/2015 19:43'!
initialize: anInteger
	"Private "
	average := DhbVector new: anInteger.
	self reset.
	^ self! !

!DhbVectorAccumulator methodsFor: 'printing' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
printOn: aStream
		"(c) Copyrights Didier BESSET, 2000, all rights reserved.
		 Initial code: 2/16/00 "
	super printOn: aStream.
	aStream space.
	count printOn: aStream.
	aStream space.
	average printOn: aStream.! !

!DhbVectorAccumulator methodsFor: 'initialization' stamp: 'StephaneDucasse 7/5/2015 19:44'!
reset
	count := 0.
	average atAllPut: 0.! !

Object subclass: #PointSeries
	instanceVariableNames: 'points'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-DHB-Numerical'!

!PointSeries class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
new
	"Create a new instance and initialize it"
	^super new initialize! !

!PointSeries methodsFor: 'public methods' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
add: anObject
	"Add a point to the receiver"
	self primitiveAdd: anObject.
	self changed: self changedSymbol.
	^anObject! !

!PointSeries methodsFor: 'public methods' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
at: anInteger
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 18/3/99 "
	^points at: anInteger! !

!PointSeries methodsFor: 'public methods' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
changedSymbol
	"Answers the symbol of the event sent when the points of the receiver change"
	^#pointsChanged! !

!PointSeries methodsFor: 'public methods' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
collectPoints: aBlock
	^points collect: aBlock! !

!PointSeries methodsFor: 'public methods' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
do: aBlock
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 18/3/99 "
	self pointsDo: aBlock.! !

!PointSeries methodsFor: 'public methods' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
firstPoint
	"Answers the first point stored in the receiver"
	^self at: 1! !

!PointSeries methodsFor: 'public methods' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialize
	"Create the point collection"
	points := SortedCollection sortBlock: self sortBlock.
	^self! !

!PointSeries methodsFor: 'public methods' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
isEmpty
		"
		 (c) Copyrights Didier BESSET, 1998, all rights reserved.
		 Initial code: 28/9/98 "
	^points isEmpty! !

!PointSeries methodsFor: 'public methods' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
notEmpty
	^points notEmpty! !

!PointSeries methodsFor: 'public methods' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
pointCollection
		" Answer the collection of points.
		 (c) Copyrights Didier BESSET, 1998, all rights reserved.
		 Initial code: 28/9/98 "
	^self collectPoints: [ :each | each]! !

!PointSeries methodsFor: 'public methods' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
pointsDo: aBlock
	points do: aBlock.! !

!PointSeries methodsFor: 'privateMethods' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
primitiveAdd: aPoint
	"Private - Add a point to the receiver"
	points add: aPoint.! !

!PointSeries methodsFor: 'privateMethods' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
primitiveRemove: aPoint
	"Private - Removes a point from the receiver"
	points remove: aPoint.! !

!PointSeries methodsFor: 'public methods' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
remove: anObject
	"Add a point to the receiver"
	self primitiveRemove: anObject.
	self changed: self changedSymbol.
	^anObject! !

!PointSeries methodsFor: 'public methods' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
size
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 18/3/99 "
	^points size! !

!PointSeries methodsFor: 'public methods' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
sort
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 27/5/99 "
	points := ( points asSortedCollection: self sortBlock) asOrderedCollection.! !

!PointSeries methodsFor: 'privateMethods' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
sortBlock
		"Private -
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 27/5/99 "
	^[ :a :b | a x < b x]! !
